@@ -1,8 +1,8 @@
 # --
-# Kernel/Modules/AgentTicketEmail.pm - to compose inital email to customer
+# Kernel/Modules/AgentTicketEmail.pm - to compose initial email to customer
 # Copyright (C) 2001-2007 OTRS GmbH, http://otrs.org/
 # --
-# $Id: AgentTicketEmail.pm,v 1.24.2.2 2007/03/12 11:52:10 mh Exp $
+# $Id: AgentTicketEmail.pm,v 1.41 2007/06/18 09:33:57 martin Exp $
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
 # the enclosed file COPYING for license information (GPL). If you
@@ -20,7 +20,7 @@ use Kernel::System::State;
 use Mail::Address;
 
 use vars qw($VERSION);
-$VERSION = '$Revision: 1.24.2.2 $';
+$VERSION = '$Revision: 1.41 $';
 $VERSION =~ s/^\$.*:\W(.*)\W.+?$/$1/;
 
 sub new {
@@ -43,8 +43,6 @@ sub new {
             $Self->{LayoutObject}->FatalError(Message => "Got no $_!");
         }
     }
-    # anyway, we need to check the email syntax (removed it, because the admins should configure it)
-#    $Self->{ConfigObject}->Set(Key => 'CheckEmailAddresses', Value => 1);
     # needed objects
     $Self->{SystemAddress} = Kernel::System::SystemAddress->new(%Param);
     $Self->{CustomerUserObject} = Kernel::System::CustomerUser->new(%Param);
@@ -80,6 +78,7 @@ sub Run {
     my %GetParam = ();
     foreach (qw(AttachmentUpload
         Year Month Day Hour Minute To Cc Bcc TimeUnits PriorityID Subject Body
+        TypeID ServiceID SLAID
         AttachmentDelete1 AttachmentDelete2 AttachmentDelete3 AttachmentDelete4
         AttachmentDelete5 AttachmentDelete6 AttachmentDelete7 AttachmentDelete8
         AttachmentDelete9 AttachmentDelete10 AttachmentDelete11 AttachmentDelete12
@@ -89,19 +88,23 @@ sub Run {
     }
     # get ticket free text params
     foreach (1..16) {
-        $GetParam{"TicketFreeKey$_"} =  $Self->{ParamObject}->GetParam(Param => "TicketFreeKey$_");
-        $GetParam{"TicketFreeText$_"} =  $Self->{ParamObject}->GetParam(Param => "TicketFreeText$_");
+        $GetParam{"TicketFreeKey$_"} = $Self->{ParamObject}->GetParam(Param => "TicketFreeKey$_");
+        $GetParam{"TicketFreeText$_"} = $Self->{ParamObject}->GetParam(Param => "TicketFreeText$_");
     }
-    # get ticket free text params
-    foreach (1..2) {
-        foreach my $Type (qw(Year Month Day Hour Minute)) {
-            $GetParam{"TicketFreeTime".$_.$Type} =  $Self->{ParamObject}->GetParam(Param => "TicketFreeTime".$_.$Type);
+    # get ticket free time params
+    foreach (1..6) {
+        foreach my $Type (qw(Used Year Month Day Hour Minute)) {
+            $GetParam{"TicketFreeTime".$_.$Type} = $Self->{ParamObject}->GetParam(Param => "TicketFreeTime".$_.$Type);
+        }
+        $GetParam{'TicketFreeTime'.$_.'Optional'} = $Self->{ConfigObject}->Get('TicketFreeTimeOptional'.$_) || 0;
+        if (!$Self->{ConfigObject}->Get('TicketFreeTimeOptional'.$_)) {
+            $GetParam{'TicketFreeTime'.$_.'Used'} = 1;
         }
     }
     # get article free text params
     foreach (1..3) {
-        $GetParam{"ArticleFreeKey$_"} =  $Self->{ParamObject}->GetParam(Param => "ArticleFreeKey$_");
-        $GetParam{"ArticleFreeText$_"} =  $Self->{ParamObject}->GetParam(Param => "ArticleFreeText$_");
+        $GetParam{"ArticleFreeKey$_"} = $Self->{ParamObject}->GetParam(Param => "ArticleFreeKey$_");
+        $GetParam{"ArticleFreeText$_"} = $Self->{ParamObject}->GetParam(Param => "ArticleFreeText$_");
     }
 
     if (!$Self->{Subaction} || $Self->{Subaction} eq 'Created') {
@@ -119,14 +122,12 @@ sub Run {
                     Link => '$Env{"Baselink"}Action=AgentTicketZoom&TicketID='.$Ticket{TicketID},
                 );
             }
-            # --
             # get split article if given
-            # --
             # get default selections
             my %TicketFreeDefault = ();
             foreach (1..16) {
-                $TicketFreeDefault{'TicketFreeKey'.$_} = $Self->{ConfigObject}->Get('TicketFreeKey'.$_.'::DefaultSelection');
-                $TicketFreeDefault{'TicketFreeText'.$_} = $Self->{ConfigObject}->Get('TicketFreeText'.$_.'::DefaultSelection');
+                $TicketFreeDefault{'TicketFreeKey'.$_} = $GetParam{'TicketFreeKey'.$_} || $Self->{ConfigObject}->Get('TicketFreeKey'.$_.'::DefaultSelection');
+                $TicketFreeDefault{'TicketFreeText'.$_} = $GetParam{'TicketFreeText'.$_} || $Self->{ConfigObject}->Get('TicketFreeText'.$_.'::DefaultSelection');
             }
             # get free text config options
             my %TicketFreeText = ();
@@ -192,25 +193,23 @@ sub Run {
             if (ref($Self->{ConfigObject}->Get('Ticket::Frontend::ArticleComposeModule')) eq 'HASH') {
                 my %Jobs = %{$Self->{ConfigObject}->Get('Ticket::Frontend::ArticleComposeModule')};
                 foreach my $Job (sort keys %Jobs) {
-                        # load module
-                        if ($Self->{MainObject}->Require($Jobs{$Job}->{Module})) {
-                            my $Object = $Jobs{$Job}->{Module}->new(
-                                %{$Self},
-                                Debug => $Self->{Debug},
-                            );
-                            # get params
-                            my %GetParam;
-                            foreach ($Object->Option(%GetParam, Config => $Jobs{$Job})) {
-                                $GetParam{$_} = $Self->{ParamObject}->GetParam(Param => $_);
-                            }
-                            # run module
-                            $Object->Run(%GetParam, Config => $Jobs{$Job});
-                            # get errors
-#                            %Error = (%Error, $Object->Error(%GetParam, Config => $Jobs{$Job}));
-                        }
-                        else {
-                            return $Self->{LayoutObject}->FatalError();
+                    # load module
+                    if ($Self->{MainObject}->Require($Jobs{$Job}->{Module})) {
+                        my $Object = $Jobs{$Job}->{Module}->new(
+                            %{$Self},
+                            Debug => $Self->{Debug},
+                        );
+                        # get params
+                        my %GetParam;
+                        foreach ($Object->Option(%GetParam, Config => $Jobs{$Job})) {
+                            $GetParam{$_} = $Self->{ParamObject}->GetParam(Param => $_);
                         }
+                        # run module
+                        $Object->Run(%GetParam, Config => $Jobs{$Job});
+                    }
+                    else {
+                        return $Self->{LayoutObject}->FatalError();
+                    }
                 }
             }
             # html output
@@ -218,13 +217,18 @@ sub Run {
                 QueueID => $Self->{QueueID},
                 NextStates => $Self->_GetNextStates(QueueID => 1),
                 Priorities => $Self->_GetPriorities(QueueID => 1),
+                Types => $Self->_GetTypes(QueueID => 1),
+                Services => $Self->_GetServices(
+                    QueueID => 1,
+                ),
+                SLAs => $Self->_GetSLAs(QueueID => 1, %GetParam),
                 Users => $Self->_GetUsers(),
                 FromList => $Self->_GetTos(),
                 To => '',
                 Subject => $Self->{LayoutObject}->Output(Template => $Self->{Config}->{Subject}),
                 Body => $Self->{LayoutObject}->Output(Template => $Self->{Config}->{Body}),
                 CustomerID => '',
-                CustomerUser =>  '',
+                CustomerUser => '',
                 CustomerData => {},
                 %TicketFreeTextHTML,
                 %TicketFreeTimeHTML,
@@ -248,20 +252,8 @@ sub Run {
             );
         }
         my $NextState = $StateData{Name};
+        my $NewResponsibleID = $Self->{ParamObject}->GetParam(Param => 'NewResponsibleID') || '';
         my $NewUserID = $Self->{ParamObject}->GetParam(Param => 'NewUserID') || '';
-        my $Lock = $Self->{ParamObject}->GetParam(Param => 'Lock') || '';
-        if ($Lock) {
-            $Self->{ConfigObject}->Set(
-                Key => 'EmailDefaultNewLock',
-                Value => $Lock,
-            );
-        }
-        if ($NewUserID) {
-            $Self->{ConfigObject}->Set(
-                Key => 'EmailDefaultNewLock',
-                Value => 'lock',
-            );
-        }
         my $Dest = $Self->{ParamObject}->GetParam(Param => 'Dest') || '';
         my ($NewQueueID, $From) = split(/\|\|/, $Dest);
         my $AllUsers = $Self->{ParamObject}->GetParam(Param => 'AllUsers') || '';
@@ -277,7 +269,7 @@ sub Run {
             $Signature = $Self->{QueueObject}->GetSignature(QueueID => $NewQueueID);
             $Signature =~ s/<OTRS_FIRST_NAME>/$Self->{UserFirstname}/g;
             $Signature =~ s/<OTRS_LAST_NAME>/$Self->{UserLastname}/g;
-            # replace user staff
+            # current user
             my %User = $Self->{UserObject}->GetUserData(
                 UserID => $Self->{UserID},
                 Cached => 1,
@@ -285,10 +277,19 @@ sub Run {
             foreach my $UserKey (keys %User) {
                 if ($User{$UserKey}) {
                     $Signature =~ s/<OTRS_Agent_$UserKey>/$User{$UserKey}/gi;
+                    $Signature =~ s/<OTRS_CURRENT_$UserKey>/$User{$UserKey}/gi;
                 }
             }
-            # cleanup all not needed <OTRS_TICKET_ tags
+            # replace other needed stuff
+            $Signature =~ s/<OTRS_FIRST_NAME>/$Self->{UserFirstname}/g;
+            $Signature =~ s/<OTRS_LAST_NAME>/$Self->{UserLastname}/g;
+            # cleanup
             $Signature =~ s/<OTRS_Agent_.+?>/-/gi;
+            $Signature =~ s/<OTRS_CURRENT_.+?>/-/gi;
+
+            # replace config options
+            $Signature =~ s{<OTRS_CONFIG_(.+?)>}{$Self->{ConfigObject}->Get($1)}egx;
+            $Signature =~ s/<OTRS_CONFIG_.+?>/-/gi;
         }
         my $CustomerUser = $Self->{ParamObject}->GetParam(Param => 'CustomerUser') ||
             $Self->{ParamObject}->GetParam(Param => 'PreSelectedCustomerUser') ||
@@ -303,6 +304,11 @@ sub Run {
             $AllUsers = 1;
             $ExpandCustomerName = 3;
         }
+        my $ResponsibleAll = $Self->{ParamObject}->GetParam(Param => 'ResponsibleAllRefresh');
+        if ($ResponsibleAll) {
+            $ResponsibleAll = 1;
+            $ExpandCustomerName = 3;
+        }
         if ($Self->{ParamObject}->GetParam(Param => 'ClearTo')) {
             $GetParam{To} = '';
             $ExpandCustomerName = 3;
@@ -333,6 +339,10 @@ sub Run {
                 QueueID => $NewQueueID || 0,
                 UserID => $Self->{UserID},
             );
+            # check required FreeTextField (if configured)
+            if ($Self->{Config}{'TicketFreeText'}{$_} == 2 && $GetParam{"TicketFreeText$_"} eq '' && $ExpandCustomerName == 0) {
+                $Error{"TicketFreeTextField$_ invalid"} = 'invalid';
+            }
         }
         my %TicketFreeTextHTML = $Self->{LayoutObject}->AgentFreeText(
             Config => \%TicketFreeText,
@@ -501,6 +511,9 @@ sub Run {
                 $Error{"Date invalid"} = 'invalid';
             }
         }
+        if ($Self->{ConfigObject}->Get('Ticket::Service') && $GetParam{SLAID} && !$GetParam{ServiceID}) {
+            $Error{"Service invalid"} = 'invalid';
+        }
         # run compose modules
         my %ArticleParam = ();
         if (ref($Self->{ConfigObject}->Get('Ticket::Frontend::ArticleComposeModule')) eq 'HASH') {
@@ -530,21 +543,25 @@ sub Run {
         }
 
         if (%Error) {
-            # --
             # header
-            # --
             $Output .= $Self->{LayoutObject}->Header();
             $Output .= $Self->{LayoutObject}->NavigationBar();
-            # --
             # html output
-            # --
             $Output .= $Self->_MaskEmailNew(
                 QueueID => $Self->{QueueID},
                 Users => $Self->_GetUsers(QueueID => $NewQueueID, AllUsers => $AllUsers),
                 UserSelected => $NewUserID,
+                ResponsibleUsers => $Self->_GetUsers(QueueID => $NewQueueID, AllUsers => $ResponsibleAll),
+                ResponsibleUsersSelected => $NewResponsibleID,
                 NextStates => $Self->_GetNextStates(QueueID => $NewQueueID || 1),
                 NextState => $NextState,
                 Priorities => $Self->_GetPriorities(QueueID => $NewQueueID || 1),
+                Types => $Self->_GetTypes(QueueID => $NewQueueID || 1),
+                Services => $Self->_GetServices(
+                    CustomerUserID => $CustomerUser || '',
+                    QueueID => $NewQueueID || 1,
+                ),
+                SLAs => $Self->_GetSLAs(QueueID => $NewQueueID || 1, %GetParam),
                 CustomerID => $Self->{LayoutObject}->Ascii2Html(Text => $CustomerID),
                 CustomerUser => $CustomerUser,
                 CustomerData => \%CustomerData,
@@ -658,10 +675,13 @@ sub Run {
             QueueID => $NewQueueID,
             Subject => $GetParam{Subject},
             Lock => 'unlock',
+            TypeID => $GetParam{TypeID} || '',
+            ServiceID => $GetParam{ServiceID} || '',
+            SLAID => $GetParam{SLAID} || '',
             StateID => $NextStateID,
             PriorityID => $GetParam{PriorityID},
             OwnerID => $Self->{UserID},
-            CustomerNo => $CustomerID,
+            CustomerID => $CustomerID,
             CustomerUser => $SelectedCustomerUser,
             UserID => $Self->{UserID},
         );
@@ -678,14 +698,30 @@ sub Run {
             }
         }
         # set ticket free time
-        foreach (1..2) {
+        foreach (1..6) {
             if (defined($GetParam{"TicketFreeTime".$_."Year"}) &&
                 defined($GetParam{"TicketFreeTime".$_."Month"}) &&
                 defined($GetParam{"TicketFreeTime".$_."Day"}) &&
                 defined($GetParam{"TicketFreeTime".$_."Hour"}) &&
-                defined($GetParam{"TicketFreeTime".$_."Minute"})) {
+                defined($GetParam{"TicketFreeTime".$_."Minute"})
+            ) {
+                my %Time;
+                $Time{"TicketFreeTime".$_."Year"} = 0;
+                $Time{"TicketFreeTime".$_."Month"} = 0;
+                $Time{"TicketFreeTime".$_."Day"} = 0;
+                $Time{"TicketFreeTime".$_."Hour"} = 0;
+                $Time{"TicketFreeTime".$_."Minute"} = 0;
+                $Time{"TicketFreeTime".$_."Secunde"} = 0;
+
+                if ($GetParam{"TicketFreeTime".$_."Used"}) {
+                    %Time = $Self->{LayoutObject}->TransfromDateSelection(
+                        %GetParam,
+                        Prefix => "TicketFreeTime".$_,
+                    );
+                }
                 $Self->{TicketObject}->TicketFreeTimeSet(
-                    %GetParam,
+                    %Time,
+                    Prefix => "TicketFreeTime",
                     TicketID => $TicketID,
                     Counter => $_,
                     UserID => $Self->{UserID},
@@ -770,6 +806,14 @@ sub Run {
                     UserID => $Self->{UserID},
                 );
             }
+            # set responsible (if new user id is given)
+            if ($NewResponsibleID) {
+                $Self->{TicketObject}->ResponsibleSet(
+                    TicketID => $TicketID,
+                    NewUserID => $NewResponsibleID,
+                    UserID => $Self->{UserID},
+                );
+            }
             # time accounting
             if ($GetParam{TimeUnits}) {
                 $Self->{TicketObject}->TicketAccountTime(
@@ -778,12 +822,6 @@ sub Run {
                     TimeUnit => $GetParam{TimeUnits},
                     UserID => $Self->{UserID},
                 );
-                # set lock
-                $Self->{TicketObject}->LockSet(
-                    TicketID => $TicketID,
-                    Lock => $Self->{ConfigObject}->Get('EmailDefaultNewLock'),
-                    UserID => $Self->{UserID},
-                );
             }
             # should i set an unlock?
             my %StateData = $Self->{StateObject}->StateGet(ID => $NextStateID);
@@ -895,6 +933,51 @@ sub _GetPriorities {
     return \%Priorities;
 }
 
+sub _GetTypes {
+    my $Self = shift;
+    my %Param = @_;
+    my %Type = ();
+    # get priority
+    if ($Param{QueueID} || $Param{TicketID}) {
+        %Type = $Self->{TicketObject}->TicketTypeList(
+            %Param,
+            Action => $Self->{Action},
+            UserID => $Self->{UserID},
+        );
+    }
+    return \%Type;
+}
+
+sub _GetServices {
+    my $Self = shift;
+    my %Param = @_;
+    my %Service = ();
+    # get priority
+    if (($Param{QueueID} || $Param{TicketID}) && $Param{CustomerUserID}) {
+        %Service = $Self->{TicketObject}->TicketServiceList(
+            %Param,
+            Action => $Self->{Action},
+            UserID => $Self->{UserID},
+        );
+    }
+    return \%Service;
+}
+
+sub _GetSLAs {
+    my $Self = shift;
+    my %Param = @_;
+    my %SLA = ();
+    # get priority
+    if ($Param{ServiceID}) {
+        %SLA = $Self->{TicketObject}->TicketSLAList(
+            %Param,
+            Action => $Self->{Action},
+            UserID => $Self->{UserID},
+        );
+    }
+    return \%SLA;
+}
+
 sub _GetTos {
     my $Self = shift;
     my %Param = @_;
@@ -956,6 +1039,11 @@ sub _MaskEmailNew {
     my $Self = shift;
     my %Param = @_;
     $Param{FormID} = $Self->{FormID};
+    # get list type
+    my $TreeView = 0;
+    if ($Self->{ConfigObject}->Get('Ticket::Frontend::ListType') eq 'tree') {
+        $TreeView = 1;
+    }
     # build string
     $Param{Users}->{''} = '-';
     $Param{'OptionStrg'} = $Self->{LayoutObject}->OptionStrgHashRef(
@@ -981,7 +1069,7 @@ sub _MaskEmailNew {
     my %NewTo = ();
     if ($Param{FromList}) {
         foreach (keys %{$Param{FromList}}) {
-             $NewTo{"$_||$Param{FromList}->{$_}"} = $Param{FromList}->{$_};
+            $NewTo{"$_||$Param{FromList}->{$_}"} = $Param{FromList}->{$_};
         }
     }
     if ($Self->{ConfigObject}->Get('Ticket::Frontend::NewQueueSelectionType') eq 'Queue') {
@@ -1014,9 +1102,57 @@ sub _MaskEmailNew {
             Data => \%Param,
         );
     }
-    # do html quoting
-    foreach (qw(From To Cc Bcc)) {
-        $Param{$_} = $Self->{LayoutObject}->Ascii2Html(Text => $Param{$_}) || '';
+    # prepare errors!
+    if ($Param{Errors}) {
+        foreach (keys %{$Param{Errors}}) {
+            $Param{$_} = "* ".$Self->{LayoutObject}->Ascii2Html(Text => $Param{Errors}->{$_});
+        }
+    }
+    # build type string
+    if ($Self->{ConfigObject}->Get('Ticket::Type')) {
+        $Param{'TypeStrg'} = $Self->{LayoutObject}->BuildSelection(
+            Data => $Param{Types},
+            Name => 'TypeID',
+            SelectedID => $Param{TypeID},
+            PossibleNone => 1,
+            Sort => 'AlphanumericValue',
+            Translation => 0,
+            OnChange => "document.compose.ExpandCustomerName.value='3'; document.compose.submit(); return false;",
+        );
+        $Self->{LayoutObject}->Block(
+            Name => 'TicketType',
+            Data => {%Param},
+        );
+    }
+    # build service string
+    if ($Self->{ConfigObject}->Get('Ticket::Service')) {
+        $Param{'ServiceStrg'} = $Self->{LayoutObject}->BuildSelection(
+            Data => $Param{Services},
+            Name => 'ServiceID',
+            SelectedID => $Param{ServiceID},
+            PossibleNone => 1,
+            TreeView => $TreeView,
+            Sort => 'TreeView',
+            Translation => 0,
+            OnChange => "document.compose.ExpandCustomerName.value='3'; document.compose.submit(); return false;",
+        );
+        $Self->{LayoutObject}->Block(
+            Name => 'TicketService',
+            Data => {%Param},
+        );
+        $Param{'SLAStrg'} = $Self->{LayoutObject}->BuildSelection(
+            Data => $Param{SLAs},
+            Name => 'SLAID',
+            SelectedID => $Param{SLAID},
+            PossibleNone => 1,
+            Sort => 'AlphanumericValue',
+            Translation => 0,
+            OnChange => "document.compose.ExpandCustomerName.value='3'; document.compose.submit(); return false;",
+        );
+        $Self->{LayoutObject}->Block(
+            Name => 'TicketSLA',
+            Data => {%Param},
+        );
     }
     # build priority string
     if (!$Param{PriorityID}) {
@@ -1034,12 +1170,6 @@ sub _MaskEmailNew {
         Format => 'DateInputFormatLong',
         DiffTime => $Self->{ConfigObject}->Get('Ticket::Frontend::PendingDiffTime') || 0,
     );
-    # prepare errors!
-    if ($Param{Errors}) {
-        foreach (keys %{$Param{Errors}}) {
-            $Param{$_} = "* ".$Self->{LayoutObject}->Ascii2Html(Text => $Param{Errors}->{$_});
-        }
-    }
     # from update
     if (!$Self->{LayoutObject}->{BrowserJavaScriptSupport}) {
         $Self->{LayoutObject}->Block(
@@ -1066,6 +1196,32 @@ sub _MaskEmailNew {
             );
         }
     }
+    # show responsible selection
+    if ($Self->{ConfigObject}->Get('Ticket::Responsible') &&
+        $Self->{ConfigObject}->Get('Ticket::Frontend::NewResponsibleSelection')) {
+        $Param{ResponsibleUsers}->{''} = '-';
+        $Param{'ResponsibleOptionStrg'} = $Self->{LayoutObject}->OptionStrgHashRef(
+            Data => $Param{ResponsibleUsers},
+            SelectedID => $Param{ResponsibleUsersSelected},
+            Name => 'NewResponsibleID',
+        );
+        $Self->{LayoutObject}->Block(
+            Name => 'ResponsibleSelection',
+            Data => \%Param,
+        );
+        if ($Self->{LayoutObject}->{BrowserJavaScriptSupport}) {
+            $Self->{LayoutObject}->Block(
+                Name => 'ResponsibleSelectionAllJS',
+                Data => { },
+            );
+        }
+        else {
+            $Self->{LayoutObject}->Block(
+                Name => 'ResponsibleSelectionAllSubmit',
+                Data => { },
+            );
+        }
+    }
     # ticket free text
     my $Count = 0;
     foreach (1..16) {
@@ -1077,6 +1233,7 @@ sub _MaskEmailNew {
                     TicketFreeKeyField => $Param{'TicketFreeKeyField'.$Count},
                     TicketFreeTextField => $Param{'TicketFreeTextField'.$Count},
                     Count => $Count,
+                    %Param,
                 },
             );
             $Self->{LayoutObject}->Block(
@@ -1089,7 +1246,7 @@ sub _MaskEmailNew {
         }
     }
     $Count = 0;
-    foreach (1..2) {
+    foreach (1..6) {
         $Count++;
         if ($Self->{Config}->{'TicketFreeTime'}->{$Count}) {
             $Self->{LayoutObject}->Block(
@@ -1163,6 +1320,18 @@ sub _MaskEmailNew {
             Data => $DataRef,
         );
     }
+    # jscript check freetextfields by submit
+    foreach my $Key (keys %{$Self->{Config}->{'TicketFreeText'}}) {
+        if ($Self->{Config}->{TicketFreeText}->{$Key} == 2) {
+            $Self->{LayoutObject}->Block(
+                Name => 'TicketFreeTextCheckJs',
+                Data => {
+                    TicketFreeTextField => "TicketFreeText$Key",
+                    TicketFreeKeyField => "TicketFreeKey$Key",
+                },
+            );
+        }
+    }
     # get output back
     return $Self->{LayoutObject}->Output(TemplateFile => 'AgentTicketEmail', Data => \%Param);
 }
