@@ -2,7 +2,7 @@
 # Kernel/Modules/AgentTicketPhone.pm - to handle phone calls
 # Copyright (C) 2001-2007 OTRS GmbH, http://otrs.org/
 # --
-# $Id: AgentTicketPhone.pm,v 1.28.2.3 2007/03/12 11:52:10 mh Exp $
+# $Id: AgentTicketPhone.pm,v 1.45 2007/07/26 17:00:42 martin Exp $
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
 # the enclosed file COPYING for license information (GPL). If you
@@ -20,7 +20,7 @@ use Kernel::System::State;
 use Mail::Address;
 
 use vars qw($VERSION);
-$VERSION = '$Revision: 1.28.2.3 $';
+$VERSION = '$Revision: 1.45 $';
 $VERSION =~ s/^\$.*:\W(.*)\W.+?$/$1/;
 
 sub new {
@@ -68,7 +68,7 @@ sub Run {
     foreach (qw(AttachmentUpload ArticleID PriorityID NewUserID
         From Subject Body NextStateID TimeUnits
         Year Month Day Hour Minute
-        NewResponsibleID ResponsibleAll OwnerAll
+        NewResponsibleID ResponsibleAll OwnerAll TypeID ServiceID SLAID
         AttachmentDelete1 AttachmentDelete2 AttachmentDelete3 AttachmentDelete4
         AttachmentDelete5 AttachmentDelete6 AttachmentDelete7 AttachmentDelete8
         AttachmentDelete9 AttachmentDelete10 AttachmentDelete11 AttachmentDelete12
@@ -78,19 +78,23 @@ sub Run {
     }
     # get ticket free text params
     foreach (1..16) {
-        $GetParam{"TicketFreeKey$_"} =  $Self->{ParamObject}->GetParam(Param => "TicketFreeKey$_");
-        $GetParam{"TicketFreeText$_"} =  $Self->{ParamObject}->GetParam(Param => "TicketFreeText$_");
+        $GetParam{"TicketFreeKey$_"} = $Self->{ParamObject}->GetParam(Param => "TicketFreeKey$_");
+        $GetParam{"TicketFreeText$_"} = $Self->{ParamObject}->GetParam(Param => "TicketFreeText$_");
     }
-    # get ticket free text params
-    foreach (1..2) {
-        foreach my $Type (qw(Year Month Day Hour Minute)) {
-            $GetParam{"TicketFreeTime".$_.$Type} =  $Self->{ParamObject}->GetParam(Param => "TicketFreeTime".$_.$Type);
+    # get ticket free time params
+    foreach (1..6) {
+        foreach my $Type (qw(Used Year Month Day Hour Minute)) {
+            $GetParam{"TicketFreeTime".$_.$Type} = $Self->{ParamObject}->GetParam(Param => "TicketFreeTime".$_.$Type);
+        }
+        $GetParam{'TicketFreeTime'.$_.'Optional'} = $Self->{ConfigObject}->Get('TicketFreeTimeOptional'.$_) || 0;
+        if (!$Self->{ConfigObject}->Get('TicketFreeTimeOptional'.$_)) {
+            $GetParam{'TicketFreeTime'.$_.'Used'} = 1;
         }
     }
     # get article free text params
     foreach (1..3) {
-        $GetParam{"ArticleFreeKey$_"} =  $Self->{ParamObject}->GetParam(Param => "ArticleFreeKey$_");
-        $GetParam{"ArticleFreeText$_"} =  $Self->{ParamObject}->GetParam(Param => "ArticleFreeText$_");
+        $GetParam{"ArticleFreeKey$_"} = $Self->{ParamObject}->GetParam(Param => "ArticleFreeKey$_");
+        $GetParam{"ArticleFreeText$_"} = $Self->{ParamObject}->GetParam(Param => "ArticleFreeText$_");
     }
 
     if (!$Self->{Subaction} || $Self->{Subaction} eq 'Created') {
@@ -116,7 +120,6 @@ sub Run {
         }
 
         # get split article if given
-
         # get ArticleID
         my %Article = ();
         my %CustomerData = ();
@@ -139,7 +142,7 @@ sub Run {
             if ($Self->{ConfigObject}->Get('Ticket::Frontend::CustomerInfoCompose')) {
                 if ($Article{CustomerUserID}) {
                     %CustomerData = $Self->{CustomerUserObject}->CustomerUserDataGet(
-                         User => $Article{CustomerUserID},
+                        User => $Article{CustomerUserID},
                     );
                 }
                 elsif ($Article{CustomerID}) {
@@ -160,8 +163,8 @@ sub Run {
         # get default selections
         my %TicketFreeDefault = ();
         foreach (1..16) {
-            $TicketFreeDefault{'TicketFreeKey'.$_} = $Self->{ConfigObject}->Get('TicketFreeKey'.$_.'::DefaultSelection');
-            $TicketFreeDefault{'TicketFreeText'.$_} = $Self->{ConfigObject}->Get('TicketFreeText'.$_.'::DefaultSelection');
+            $TicketFreeDefault{'TicketFreeKey'.$_} = $GetParam{'TicketFreeKey'.$_} || $Self->{ConfigObject}->Get('TicketFreeKey'.$_.'::DefaultSelection');
+            $TicketFreeDefault{'TicketFreeText'.$_} = $GetParam{'TicketFreeText'.$_} || $Self->{ConfigObject}->Get('TicketFreeText'.$_.'::DefaultSelection');
         }
         # get free text config options
         my %TicketFreeText = ();
@@ -227,6 +230,12 @@ sub Run {
             QueueID => $Self->{QueueID},
             NextStates => $Self->_GetNextStates(QueueID => $Self->{QueueID} || 1),
             Priorities => $Self->_GetPriorities(QueueID => $Self->{QueueID} || 1),
+            Types => $Self->_GetTypes(QueueID => $Self->{QueueID} || 1),
+            Services => $Self->_GetServices(
+                CustomerUserID => $CustomerData{CustomerUserLogin} || '',
+                QueueID => $Self->{QueueID} || 1,
+            ),
+            SLAs => $Self->_GetSLAs(QueueID => $Self->{QueueID} || 1, %GetParam),
             Users => $Self->_GetUsers(QueueID => $Self->{QueueID}),
             To => $Self->_GetTos(QueueID => $Self->{QueueID}),
             From => $Article{From},
@@ -252,18 +261,6 @@ sub Run {
             );
         }
         my $NextState = $StateData{Name} || '';
-        if ($GetParam{Lock}) {
-            $Self->{ConfigObject}->Set(
-                Key => 'PhoneDefaultNewLock',
-                Value => $GetParam{Lock},
-            );
-        }
-        if ($GetParam{NewUserID}) {
-            $Self->{ConfigObject}->Set(
-                Key => 'PhoneDefaultNewLock',
-                Value => 'lock',
-            );
-        }
         my $Dest = $Self->{ParamObject}->GetParam(Param => 'Dest') || '';
         my ($NewQueueID, $To) = split(/\|\|/, $Dest);
         my $CustomerUser = $Self->{ParamObject}->GetParam(Param => 'CustomerUser') ||
@@ -350,6 +347,10 @@ sub Run {
                 QueueID => $NewQueueID || 0,
                 UserID => $Self->{UserID},
             );
+            # check required FreeTextField (if configured)
+            if ($Self->{Config}{'TicketFreeText'}{$_} == 2 && $GetParam{"TicketFreeText$_"} eq '' && $ExpandCustomerName == 0) {
+                $Error{"TicketFreeTextField$_ invalid"} = 'invalid';
+            }
         }
         my %TicketFreeTextHTML = $Self->{LayoutObject}->AgentFreeText(
             Config => \%TicketFreeText,
@@ -359,26 +360,26 @@ sub Run {
         my %TicketFreeTimeHTML = $Self->{LayoutObject}->AgentFreeDate(
             Ticket => \%GetParam,
         );
-            # article free text
-            my %ArticleFreeText = ();
-            foreach (1..3) {
-                $ArticleFreeText{"ArticleFreeKey$_"} = $Self->{TicketObject}->ArticleFreeTextGet(
-                    TicketID => $Self->{TicketID},
-                    Type => "ArticleFreeKey$_",
-                    Action => $Self->{Action},
-                    UserID => $Self->{UserID},
-                );
-                $ArticleFreeText{"ArticleFreeText$_"} = $Self->{TicketObject}->ArticleFreeTextGet(
-                    TicketID => $Self->{TicketID},
-                    Type => "ArticleFreeText$_",
-                    Action => $Self->{Action},
-                    UserID => $Self->{UserID},
-                );
-            }
-            my %ArticleFreeTextHTML = $Self->{LayoutObject}->TicketArticleFreeText(
-                Config => \%ArticleFreeText,
-                Article => \%GetParam,
+        # article free text
+        my %ArticleFreeText = ();
+        foreach (1..3) {
+            $ArticleFreeText{"ArticleFreeKey$_"} = $Self->{TicketObject}->ArticleFreeTextGet(
+                TicketID => $Self->{TicketID},
+                Type => "ArticleFreeKey$_",
+                Action => $Self->{Action},
+                UserID => $Self->{UserID},
             );
+            $ArticleFreeText{"ArticleFreeText$_"} = $Self->{TicketObject}->ArticleFreeTextGet(
+                TicketID => $Self->{TicketID},
+                Type => "ArticleFreeText$_",
+                Action => $Self->{Action},
+                UserID => $Self->{UserID},
+            );
+        }
+        my %ArticleFreeTextHTML = $Self->{LayoutObject}->TicketArticleFreeText(
+            Config => \%ArticleFreeText,
+            Article => \%GetParam,
+        );
         # expand customer name
         my %CustomerUserData = ();
         if ($ExpandCustomerName == 1) {
@@ -481,8 +482,10 @@ sub Run {
         if (!$NewQueueID && $ExpandCustomerName == 0) {
             $Error{"Destination invalid"} = 'invalid';
         }
+        if ($Self->{ConfigObject}->Get('Ticket::Service') && $GetParam{SLAID} && !$GetParam{ServiceID}) {
+            $Error{"Service invalid"} = 'invalid';
+        }
         if (%Error) {
-
             # header
             my $Output = $Self->{LayoutObject}->Header();
             $Output .= $Self->{LayoutObject}->NavigationBar();
@@ -497,6 +500,12 @@ sub Run {
                 NextStates => $Self->_GetNextStates(QueueID => $NewQueueID || 1),
                 NextState => $NextState,
                 Priorities => $Self->_GetPriorities(QueueID => $NewQueueID || 1),
+                Types => $Self->_GetTypes(QueueID => $NewQueueID || 1),
+                Services => $Self->_GetServices(
+                    CustomerUserID => $CustomerUser || '',
+                    QueueID => $NewQueueID || 1,
+                ),
+                SLAs => $Self->_GetSLAs(QueueID => $NewQueueID || 1, %GetParam),
                 CustomerID => $Self->{LayoutObject}->Ascii2Html(Text => $CustomerID),
                 CustomerUser => $CustomerUser,
                 CustomerData => \%CustomerData,
@@ -595,8 +604,8 @@ sub Run {
                     Data => {
                         %AclAction,
                         %Article,
-                        Age => $Self->{LayoutObject}->CustomerAge(Age => $Article{Age}, Space => ' '),
-                        TicketOverTime => $Self->{LayoutObject}->CustomerAge(Age => $Article{TicketOverTime}, Space => ' '),
+                        Age => $Self->{LayoutObject}->CustomerAge(Age => $Article{Age}, Space => ' ') || '',
+                        TicketOverTime => $Self->{LayoutObject}->CustomerAge(Age => $Article{TicketOverTime}, Space => ' ') || '',
                     }
                 );
             }
@@ -609,6 +618,9 @@ sub Run {
             QueueID => $NewQueueID,
             Subject => $GetParam{Subject},
             Lock => 'unlock',
+            TypeID => $GetParam{TypeID} || '',
+            ServiceID => $GetParam{ServiceID} || '',
+            SLAID => $GetParam{SLAID} || '',
             StateID => $GetParam{NextStateID},
             PriorityID => $GetParam{PriorityID},
             OwnerID => $Self->{UserID},
@@ -629,17 +641,27 @@ sub Run {
             }
         }
         # set ticket free time
-        foreach (1..2) {
+        foreach (1..6) {
             if (defined($GetParam{"TicketFreeTime".$_."Year"}) &&
                 defined($GetParam{"TicketFreeTime".$_."Month"}) &&
                 defined($GetParam{"TicketFreeTime".$_."Day"}) &&
                 defined($GetParam{"TicketFreeTime".$_."Hour"}) &&
                 defined($GetParam{"TicketFreeTime".$_."Minute"})
             ) {
-                my %Time = $Self->{LayoutObject}->TransfromDateSelection(
-                    %GetParam,
-                    Prefix => "TicketFreeTime".$_,
-                );
+                my %Time;
+                $Time{"TicketFreeTime".$_."Year"} = 0;
+                $Time{"TicketFreeTime".$_."Month"} = 0;
+                $Time{"TicketFreeTime".$_."Day"} = 0;
+                $Time{"TicketFreeTime".$_."Hour"} = 0;
+                $Time{"TicketFreeTime".$_."Minute"} = 0;
+                $Time{"TicketFreeTime".$_."Secunde"} = 0;
+
+                if ($GetParam{"TicketFreeTime".$_."Used"}) {
+                    %Time = $Self->{LayoutObject}->TransfromDateSelection(
+                        %GetParam,
+                        Prefix => "TicketFreeTime".$_,
+                    );
+                }
                 $Self->{TicketObject}->TicketFreeTimeSet(
                     %Time,
                     Prefix => "TicketFreeTime",
@@ -855,6 +877,51 @@ sub _GetPriorities {
     return \%Priorities;
 }
 
+sub _GetTypes {
+    my $Self = shift;
+    my %Param = @_;
+    my %Type = ();
+    # get priority
+    if ($Param{QueueID} || $Param{TicketID}) {
+        %Type = $Self->{TicketObject}->TicketTypeList(
+            %Param,
+            Action => $Self->{Action},
+            UserID => $Self->{UserID},
+        );
+    }
+    return \%Type;
+}
+
+sub _GetServices {
+    my $Self = shift;
+    my %Param = @_;
+    my %Service = ();
+    # get priority
+    if (($Param{QueueID} || $Param{TicketID}) && $Param{CustomerUserID}) {
+        %Service = $Self->{TicketObject}->TicketServiceList(
+            %Param,
+            Action => $Self->{Action},
+            UserID => $Self->{UserID},
+        );
+    }
+    return \%Service;
+}
+
+sub _GetSLAs {
+    my $Self = shift;
+    my %Param = @_;
+    my %SLA = ();
+    # get priority
+    if ($Param{ServiceID}) {
+        %SLA = $Self->{TicketObject}->TicketSLAList(
+            %Param,
+            Action => $Self->{Action},
+            UserID => $Self->{UserID},
+        );
+    }
+    return \%SLA;
+}
+
 sub _GetTos {
     my $Self = shift;
     my %Param = @_;
@@ -917,6 +984,11 @@ sub _MaskPhoneNew {
     my $Self = shift;
     my %Param = @_;
     $Param{FormID} = $Self->{FormID};
+    # get list type
+    my $TreeView = 0;
+    if ($Self->{ConfigObject}->Get('Ticket::Frontend::ListType') eq 'tree') {
+        $TreeView = 1;
+    }
     # build string
     $Param{Users}->{''} = '-';
     $Param{'OptionStrg'} = $Self->{LayoutObject}->OptionStrgHashRef(
@@ -942,7 +1014,7 @@ sub _MaskPhoneNew {
     my %NewTo = ();
     if ($Param{To}) {
         foreach (keys %{$Param{To}}) {
-             $NewTo{"$_||$Param{To}->{$_}"} = $Param{To}->{$_};
+            $NewTo{"$_||$Param{To}->{$_}"} = $Param{To}->{$_};
         }
     }
     if ($Self->{ConfigObject}->Get('Ticket::Frontend::NewQueueSelectionType') eq 'Queue') {
@@ -975,6 +1047,58 @@ sub _MaskPhoneNew {
             Data => \%Param,
         );
     }
+    # prepare errors!
+    if ($Param{Errors}) {
+        foreach (keys %{$Param{Errors}}) {
+            $Param{$_} = "* ".$Self->{LayoutObject}->Ascii2Html(Text => $Param{Errors}->{$_});
+        }
+    }
+    # build type string
+    if ($Self->{ConfigObject}->Get('Ticket::Type')) {
+        $Param{'TypeStrg'} = $Self->{LayoutObject}->BuildSelection(
+            Data => $Param{Types},
+            Name => 'TypeID',
+            SelectedID => $Param{TypeID},
+            PossibleNone => 1,
+            Sort => 'AlphanumericValue',
+            Translation => 0,
+            OnChange => "document.compose.ExpandCustomerName.value='3'; document.compose.submit(); return false;",
+        );
+        $Self->{LayoutObject}->Block(
+            Name => 'TicketType',
+            Data => {%Param},
+        );
+    }
+    # build service string
+    if ($Self->{ConfigObject}->Get('Ticket::Service')) {
+        $Param{'ServiceStrg'} = $Self->{LayoutObject}->BuildSelection(
+            Data => $Param{Services},
+            Name => 'ServiceID',
+            SelectedID => $Param{ServiceID},
+            PossibleNone => 1,
+            TreeView => $TreeView,
+            Sort => 'TreeView',
+            Translation => 0,
+            OnChange => "document.compose.ExpandCustomerName.value='3'; document.compose.submit(); return false;",
+        );
+        $Self->{LayoutObject}->Block(
+            Name => 'TicketService',
+            Data => {%Param},
+        );
+        $Param{'SLAStrg'} = $Self->{LayoutObject}->BuildSelection(
+            Data => $Param{SLAs},
+            Name => 'SLAID',
+            SelectedID => $Param{SLAID},
+            PossibleNone => 1,
+            Sort => 'AlphanumericValue',
+            Translation => 0,
+            OnChange => "document.compose.ExpandCustomerName.value='3'; document.compose.submit(); return false;",
+        );
+        $Self->{LayoutObject}->Block(
+            Name => 'TicketSLA',
+            Data => {%Param},
+        );
+    }
     # build priority string
     if (!$Param{PriorityID}) {
         $Param{Priority} = $Self->{Config}->{Priority};
@@ -991,12 +1115,6 @@ sub _MaskPhoneNew {
         Format => 'DateInputFormatLong',
         DiffTime => $Self->{ConfigObject}->Get('Ticket::Frontend::PendingDiffTime') || 0,
     );
-    # prepare errors!
-    if ($Param{Errors}) {
-        foreach (keys %{$Param{Errors}}) {
-            $Param{$_} = "* ".$Self->{LayoutObject}->Ascii2Html(Text => $Param{Errors}->{$_});
-        }
-    }
     # to update
     if (!$Self->{LayoutObject}->{BrowserJavaScriptSupport}) {
         $Self->{LayoutObject}->Block(
@@ -1060,7 +1178,7 @@ sub _MaskPhoneNew {
                     TicketFreeKeyField => $Param{'TicketFreeKeyField'.$Count},
                     TicketFreeTextField => $Param{'TicketFreeTextField'.$Count},
                     Count => $Count,
-
+                    %Param,
                 },
             );
             $Self->{LayoutObject}->Block(
@@ -1073,7 +1191,7 @@ sub _MaskPhoneNew {
         }
     }
     $Count = 0;
-    foreach (1..2) {
+    foreach (1..6) {
         $Count++;
         if ($Self->{Config}->{'TicketFreeTime'}->{$Count}) {
             $Self->{LayoutObject}->Block(
@@ -1140,6 +1258,18 @@ sub _MaskPhoneNew {
             Data => $DataRef,
         );
     }
+    # jscript check freetextfields by submit
+    foreach my $Key (keys %{$Self->{Config}->{TicketFreeText}}) {
+        if ($Self->{Config}->{TicketFreeText}->{$Key} == 2) {
+            $Self->{LayoutObject}->Block(
+                Name => 'TicketFreeTextCheckJs',
+                Data => {
+                    TicketFreeTextField => "TicketFreeText$Key",
+                    TicketFreeKeyField => "TicketFreeKey$Key",
+                },
+            );
+        }
+    }
     # get output back
     return $Self->{LayoutObject}->Output(TemplateFile => 'AgentTicketPhone', Data => \%Param);
 }
