@@ -2,7 +2,7 @@
 # Kernel/Modules/AgentTicketPrint.pm - to get a closer view
 # Copyright (C) 2001-2007 OTRS GmbH, http://otrs.org/
 # --
-# $Id: AgentTicketPrint.pm,v 1.32.2.3 2007/04/03 23:15:33 mh Exp $
+# $Id: AgentTicketPrint.pm,v 1.42 2007/07/26 14:22:43 mh Exp $
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
 # the enclosed file COPYING for license information (GPL). If you
@@ -17,7 +17,7 @@ use Kernel::System::LinkObject;
 use Kernel::System::PDF;
 
 use vars qw($VERSION);
-$VERSION = '$Revision: 1.32.2.3 $';
+$VERSION = '$Revision: 1.42 $';
 $VERSION =~ s/^\$.*:\W(.*)\W.+?$/$1/;
 
 sub new {
@@ -164,6 +164,7 @@ sub Run {
         # create new pdf document
         $Self->{PDFObject}->DocumentNew(
             Title => $Self->{ConfigObject}->Get('Product') . ': ' . $Title,
+            Encode => $Self->{LayoutObject}->{UserCharset},
         );
         # create first pdf page
         $Self->{PDFObject}->PageNew(
@@ -276,63 +277,98 @@ sub _PDFOutputTicketInfos {
     my %Ticket = %{$Param{TicketData}};
     my %UserInfo = %{$Param{UserData}};
     my %Page = %{$Param{PageData}};
+    # create left table
+    my $TableLeft = [
+        {
+            Key => $Self->{LayoutObject}->{LanguageObject}->Get('State') . ':',
+            Value => $Self->{LayoutObject}->{LanguageObject}->Get($Ticket{State}),
+        },
+        {
+            Key => $Self->{LayoutObject}->{LanguageObject}->Get('Priority') . ':',
+            Value => $Self->{LayoutObject}->{LanguageObject}->Get($Ticket{Priority}),
+        },
+        {
+            Key => $Self->{LayoutObject}->{LanguageObject}->Get('Queue') . ':',
+            Value => $Ticket{Queue},
+        },
+        {
+            Key => $Self->{LayoutObject}->{LanguageObject}->Get('Lock') . ':',
+            Value => $Self->{LayoutObject}->{LanguageObject}->Get($Ticket{Lock}),
+        },
+        {
+            Key => $Self->{LayoutObject}->{LanguageObject}->Get('CustomerID') . ':',
+            Value => $Ticket{CustomerID},
+        },
+        {
+            Key => $Self->{LayoutObject}->{LanguageObject}->Get('Owner') . ':',
+            Value => $Ticket{Owner} . ' (' . $UserInfo{UserFirstname} . ' ' . $UserInfo{UserLastname} . ')',
+        },
+    ];
+    # add type row, if feature is enabled
+    if ($Self->{ConfigObject}->Get('Ticket::Type')) {
+        my $RowType = {
+            Key => $Self->{LayoutObject}->{LanguageObject}->Get('Type') . ':',
+            Value => $Ticket{Type},
+        };
+        push(@{$TableLeft}, $RowType);
+    }
+    # add service and sla row, if feature is enabled
+    if ($Self->{ConfigObject}->Get('Ticket::Service')) {
+        my $RowService = {
+            Key => $Self->{LayoutObject}->{LanguageObject}->Get('Service') . ':',
+            Value => $Ticket{Service},
+        };
+        push(@{$TableLeft}, $RowService);
+        my $RowSLA = {
+            Key => $Self->{LayoutObject}->{LanguageObject}->Get('SLA') . ':',
+            Value => $Ticket{SLA},
+        };
+        push(@{$TableLeft}, $RowSLA);
+    }
+    # create right table
+    my $TableRight = [
+        {
+            Key => $Self->{LayoutObject}->{LanguageObject}->Get('Age') . ':',
+            Value => $Self->{LayoutObject}->{LanguageObject}->Get($Ticket{Age}),
+        },
+        {
+            Key => $Self->{LayoutObject}->{LanguageObject}->Get('Created') . ':',
+            Value => $Self->{LayoutObject}->Output(
+                Template => '$TimeLong{"$Data{"Created"}"}',
+                Data => \%Ticket,
+            ),
+        },
+        {
+            Key => $Self->{LayoutObject}->{LanguageObject}->Get('Accounted time') . ':',
+            Value => $Ticket{TicketTimeUnits},
+        },
+        {
+            Key => $Self->{LayoutObject}->{LanguageObject}->Get('Escalation in') . ':',
+            Value => $Ticket{TicketOverTime},
+        },
+        {
+            Key => $Self->{LayoutObject}->{LanguageObject}->Get('Pending till') . ':',
+            Value => $Ticket{PendingUntil},
+        },
+    ];
+
+    my $Rows = @{$TableLeft};
+    if (@{$TableRight} > $Rows) {
+        $Rows = @{$TableRight};
+    }
+
     my %TableParam;
-    $TableParam{CellData}[0][0]{Content} = $Self->{LayoutObject}->{LanguageObject}->Get('State') . ':';
-    $TableParam{CellData}[0][0]{Font} = 'HelveticaBold';
-    $TableParam{CellData}[0][1]{Content} = $Self->{LayoutObject}->{LanguageObject}->Get($Ticket{State});
-    $TableParam{CellData}[0][2]{Content} = ' ';
-    $TableParam{CellData}[0][2]{BackgroundColor} = '#FFFFFF';
-    $TableParam{CellData}[0][3]{Content} = $Self->{LayoutObject}->{LanguageObject}->Get('Age') . ':';
-    $TableParam{CellData}[0][3]{Font} = 'HelveticaBold';
-    $TableParam{CellData}[0][4]{Content} = $Self->{LayoutObject}->{LanguageObject}->Get($Ticket{Age});
-
-    $TableParam{CellData}[1][0]{Content} = $Self->{LayoutObject}->{LanguageObject}->Get('Priority') . ':';
-    $TableParam{CellData}[1][0]{Font} = 'HelveticaBold';
-    $TableParam{CellData}[1][1]{Content} = $Self->{LayoutObject}->{LanguageObject}->Get($Ticket{Priority});
-    $TableParam{CellData}[1][2]{Content} = ' ';
-    $TableParam{CellData}[1][2]{BackgroundColor} = '#FFFFFF';
-    $TableParam{CellData}[1][3]{Content} = $Self->{LayoutObject}->{LanguageObject}->Get('Created') . ':';
-    $TableParam{CellData}[1][3]{Font} = 'HelveticaBold';
-    $TableParam{CellData}[1][4]{Content} =
-        $Self->{LayoutObject}->Output(
-            Template => '$TimeLong{"$Data{"Created"}"}',
-            Data => \%Ticket,
-        );
-    $TableParam{CellData}[2][0]{Content} = $Self->{LayoutObject}->{LanguageObject}->Get('Queue') . ':';
-    $TableParam{CellData}[2][0]{Font} = 'HelveticaBold';
-    $TableParam{CellData}[2][1]{Content} = $Ticket{Queue};
-    $TableParam{CellData}[2][2]{Content} = ' ';
-    $TableParam{CellData}[2][2]{BackgroundColor} = '#FFFFFF';
-    $TableParam{CellData}[2][3]{Content} = $Self->{LayoutObject}->{LanguageObject}->Get('Accounted time') . ':';
-    $TableParam{CellData}[2][3]{Font} = 'HelveticaBold';
-    $TableParam{CellData}[2][4]{Content} = $Ticket{TicketTimeUnits};
-
-    $TableParam{CellData}[3][0]{Content} = $Self->{LayoutObject}->{LanguageObject}->Get('Lock') . ':';
-    $TableParam{CellData}[3][0]{Font} = 'HelveticaBold';
-    $TableParam{CellData}[3][1]{Content} = $Self->{LayoutObject}->{LanguageObject}->Get($Ticket{Lock});
-    $TableParam{CellData}[3][2]{Content} = ' ';
-    $TableParam{CellData}[3][2]{BackgroundColor} = '#FFFFFF';
-    $TableParam{CellData}[3][3]{Content} = $Self->{LayoutObject}->{LanguageObject}->Get('Escalation in') . ':';
-    $TableParam{CellData}[3][3]{Font} = 'HelveticaBold';
-    $TableParam{CellData}[3][4]{Content} = $Ticket{TicketOverTime};
-
-    $TableParam{CellData}[4][0]{Content} = $Self->{LayoutObject}->{LanguageObject}->Get('CustomerID') . ':';
-    $TableParam{CellData}[4][0]{Font} = 'HelveticaBold';
-    $TableParam{CellData}[4][1]{Content} = $Ticket{CustomerID};
-    $TableParam{CellData}[4][2]{Content} = ' ';
-    $TableParam{CellData}[4][2]{BackgroundColor} = '#FFFFFF';
-    $TableParam{CellData}[4][3]{Content} = $Self->{LayoutObject}->{LanguageObject}->Get('Pending till') . ':';
-    $TableParam{CellData}[4][3]{Font} = 'HelveticaBold';
-    $TableParam{CellData}[4][4]{Content} = $Ticket{PendingUntil};
-
-    $TableParam{CellData}[5][0]{Content} = $Self->{LayoutObject}->{LanguageObject}->Get('Owner') . ':';
-    $TableParam{CellData}[5][0]{Font} = 'HelveticaBold';
-    $TableParam{CellData}[5][1]{Content} = $Ticket{Owner} . ' (' . $UserInfo{UserFirstname} . ' ' . $UserInfo{UserLastname} . ')';
-    $TableParam{CellData}[5][2]{Content} = ' ';
-    $TableParam{CellData}[5][2]{BackgroundColor} = '#FFFFFF';
-    $TableParam{CellData}[5][3]{Content} = ' ';
-    $TableParam{CellData}[5][3]{Font} = 'HelveticaBold';
-    $TableParam{CellData}[5][4]{Content} = ' ';
+    foreach my $Row (1..$Rows) {
+        $Row--;
+        $TableParam{CellData}[$Row][0]{Content} = $TableLeft->[$Row]->{Key};
+        $TableParam{CellData}[$Row][0]{Font} = 'ProportionalBold';
+        $TableParam{CellData}[$Row][1]{Content} = $TableLeft->[$Row]->{Value};
+        $TableParam{CellData}[$Row][2]{Content} = ' ';
+        $TableParam{CellData}[$Row][2]{BackgroundColor} = '#FFFFFF';
+        $TableParam{CellData}[$Row][3]{Content} = $TableRight->[$Row]->{Key};
+        $TableParam{CellData}[$Row][3]{Font} = 'ProportionalBold';
+        $TableParam{CellData}[$Row][4]{Content} = $TableRight->[$Row]->{Value};
+    }
 
     $TableParam{ColumnData}[0]{Width} = 80;
     $TableParam{ColumnData}[1]{Width} = 170.5;
@@ -367,6 +403,7 @@ sub _PDFOutputTicketInfos {
             $Page{PageCount}++;
         }
     }
+    return 1;
 }
 
 sub _PDFOutputLinkedObjects {
@@ -409,21 +446,21 @@ sub _PDFOutputLinkedObjects {
         if ($LONormal) {
             $TableParam{CellData}[$Row][0]{Content} =
                 $Self->{LayoutObject}->{LanguageObject}->Get('Normal') . ':';
-            $TableParam{CellData}[$Row][0]{Font} = 'HelveticaBold';
+            $TableParam{CellData}[$Row][0]{Font} = 'ProportionalBold';
             $TableParam{CellData}[$Row][1]{Content} = $LONormal;
             $Row++;
         }
         if ($LOParent) {
             $TableParam{CellData}[$Row][0]{Content} =
                 $Self->{LayoutObject}->{LanguageObject}->Get('Parent') . ':';
-            $TableParam{CellData}[$Row][0]{Font} = 'HelveticaBold';
+            $TableParam{CellData}[$Row][0]{Font} = 'ProportionalBold';
             $TableParam{CellData}[$Row][1]{Content} = $LOParent;
             $Row++;
         }
         if ($LOChild) {
             $TableParam{CellData}[$Row][0]{Content} =
                 $Self->{LayoutObject}->{LanguageObject}->Get('Child') . ':';
-            $TableParam{CellData}[$Row][0]{Font} = 'HelveticaBold';
+            $TableParam{CellData}[$Row][0]{Font} = 'ProportionalBold';
             $TableParam{CellData}[$Row][1]{Content} = $LOChild;
             $Row++;
         }
@@ -440,7 +477,7 @@ sub _PDFOutputLinkedObjects {
             Text => $Self->{LayoutObject}->{LanguageObject}->Get('Linked Objects'),
             Height => 7,
             Type => 'Cut',
-            Font => 'HelveticaBoldItalic',
+            Font => 'ProportionalBoldItalic',
             FontSize => 7,
             Color => '#666666',
         );
@@ -477,6 +514,7 @@ sub _PDFOutputLinkedObjects {
             }
         }
     }
+    return 1;
 }
 
 sub _PDFOutputTicketFreeText {
@@ -499,7 +537,7 @@ sub _PDFOutputTicketFreeText {
     foreach (1..16) {
         if ($Ticket{"TicketFreeText$_"} ne "") {
             $TableParam{CellData}[$Row][0]{Content} = $Ticket{"TicketFreeKey$_"} . ':';
-            $TableParam{CellData}[$Row][0]{Font} = 'HelveticaBold';
+            $TableParam{CellData}[$Row][0]{Font} = 'ProportionalBold';
             $TableParam{CellData}[$Row][1]{Content} = $Ticket{"TicketFreeText$_"};
 
             $Row++;
@@ -521,7 +559,7 @@ sub _PDFOutputTicketFreeText {
             Text => $Self->{LayoutObject}->{LanguageObject}->Get('TicketFreeText'),
             Height => 7,
             Type => 'Cut',
-            Font => 'HelveticaBoldItalic',
+            Font => 'ProportionalBoldItalic',
             FontSize => 7,
             Color => '#666666',
         );
@@ -559,6 +597,7 @@ sub _PDFOutputTicketFreeText {
             }
         }
     }
+    return 1;
 }
 
 sub _PDFOutputTicketFreeTime {
@@ -578,13 +617,13 @@ sub _PDFOutputTicketFreeTime {
     my %TableParam;
     my $Row = 0;
     # generate table
-    foreach (1..2) {
+    foreach (1..6) {
         if ($Ticket{"TicketFreeTime$_"}) {
             my $TicketFreeTimeKey = $Self->{ConfigObject}->Get('TicketFreeTimeKey'.$_) || '';
             my $TicketFreeTime = $Ticket{"TicketFreeTime$_"};
 
             $TableParam{CellData}[$Row][0]{Content} = $TicketFreeTimeKey . ':';
-            $TableParam{CellData}[$Row][0]{Font} = 'HelveticaBold';
+            $TableParam{CellData}[$Row][0]{Font} = 'ProportionalBold';
             $TableParam{CellData}[$Row][1]{Content} = $Self->{LayoutObject}->Output(
                 Template => '$TimeLong{"$Data{"TicketFreeTime"}"}',
                 Data => {
@@ -611,7 +650,7 @@ sub _PDFOutputTicketFreeTime {
             Text => $Self->{LayoutObject}->{LanguageObject}->Get('TicketFreeTime'),
             Height => 7,
             Type => 'Cut',
-            Font => 'HelveticaBoldItalic',
+            Font => 'ProportionalBoldItalic',
             FontSize => 7,
             Color => '#666666',
         );
@@ -649,6 +688,7 @@ sub _PDFOutputTicketFreeTime {
             }
         }
     }
+    return 1;
 }
 
 sub _PDFOutputCustomerInfos {
@@ -666,12 +706,19 @@ sub _PDFOutputCustomerInfos {
     my %Page = %{$Param{PageData}};
     my %TableParam;
     my $Row = 0;
-    my $Map = $CustomerData{Config}{Map};
+    my $Map = $CustomerData{Config}->{Map};
+    # check if customer company support is enabled
+    if ($CustomerData{Config}->{CustomerCompanySupport}) {
+        my $Map2 = $CustomerData{CompanyConfig}->{Map};
+        if ($Map2) {
+            push (@{$Map}, @{$Map2});
+        }
+    }
     foreach my $Field (@{$Map}) {
         if (${$Field}[3] && $CustomerData{${$Field}[0]}) {
             $TableParam{CellData}[$Row][0]{Content} =
                 $Self->{LayoutObject}->{LanguageObject}->Get(${$Field}[1]) . ':';
-            $TableParam{CellData}[$Row][0]{Font} = 'HelveticaBold';
+            $TableParam{CellData}[$Row][0]{Font} = 'ProportionalBold';
             $TableParam{CellData}[$Row][1]{Content} = $CustomerData{${$Field}[0]};
 
             $Row++;
@@ -692,7 +739,7 @@ sub _PDFOutputCustomerInfos {
             Text => $Self->{LayoutObject}->{LanguageObject}->Get('Customer Infos'),
             Height => 7,
             Type => 'Cut',
-            Font => 'HelveticaBoldItalic',
+            Font => 'ProportionalBoldItalic',
             FontSize => 7,
             Color => '#666666',
         );
@@ -729,6 +776,7 @@ sub _PDFOutputCustomerInfos {
             }
         }
     }
+    return 1;
 }
 
 sub _PDFOutputArticles {
@@ -755,7 +803,7 @@ sub _PDFOutputArticles {
                 Text => $Self->{LayoutObject}->{LanguageObject}->Get('Articles'),
                 Height => 7,
                 Type => 'Cut',
-                Font => 'HelveticaBoldItalic',
+                Font => 'ProportionalBoldItalic',
                 FontSize => 7,
                 Color => '#666666',
             );
@@ -783,13 +831,13 @@ sub _PDFOutputArticles {
         foreach (qw(From To Cc Subject)) {
             if ($Article{$_}) {
                 $TableParam1{CellData}[$Row][0]{Content} = $Self->{LayoutObject}->{LanguageObject}->Get($_) . ':';
-                $TableParam1{CellData}[$Row][0]{Font} = 'HelveticaBold';
+                $TableParam1{CellData}[$Row][0]{Font} = 'ProportionalBold';
                 $TableParam1{CellData}[$Row][1]{Content} = $Article{$_};
                 $Row++;
             }
         }
         $TableParam1{CellData}[$Row][0]{Content} = $Self->{LayoutObject}->{LanguageObject}->Get('Created') . ':';
-        $TableParam1{CellData}[$Row][0]{Font} = 'HelveticaBold';
+        $TableParam1{CellData}[$Row][0]{Font} = 'ProportionalBold';
         $TableParam1{CellData}[$Row][1]{Content} =
             $Self->{LayoutObject}->Output(
                 Template => '$TimeLong{"$Data{"Created"}"}',
@@ -802,20 +850,20 @@ sub _PDFOutputArticles {
         foreach (1..3) {
             if ($Article{"ArticleFreeText$_"}) {
                 $TableParam1{CellData}[$Row][0]{Content} = $Article{"ArticleFreeKey$_"} . ':';
-                $TableParam1{CellData}[$Row][0]{Font} = 'HelveticaBold';
+                $TableParam1{CellData}[$Row][0]{Font} = 'ProportionalBold';
                 $TableParam1{CellData}[$Row][1]{Content} = $Article{"ArticleFreeText$_"};
                 $Row++;
             }
         }
 
         $TableParam1{CellData}[$Row][0]{Content} = $Self->{LayoutObject}->{LanguageObject}->Get('Type') . ':';
-        $TableParam1{CellData}[$Row][0]{Font} = 'HelveticaBold';
+        $TableParam1{CellData}[$Row][0]{Font} = 'ProportionalBold';
         $TableParam1{CellData}[$Row][1]{Content} = $Article{ArticleType};
         $Row++;
 
         if ($Attachments) {
             $TableParam1{CellData}[$Row][0]{Content} = $Self->{LayoutObject}->{LanguageObject}->Get('Attachment') . ':';
-            $TableParam1{CellData}[$Row][0]{Font} = 'HelveticaBold';
+            $TableParam1{CellData}[$Row][0]{Font} = 'ProportionalBold';
             chomp($Attachments);
             $TableParam1{CellData}[$Row][1]{Content} = $Attachments;
         }
@@ -860,7 +908,7 @@ sub _PDFOutputArticles {
         $TableParam2{CellData}[0][0]{Content} = $Article{Body} || ' ';
         $TableParam2{Type} = 'Cut';
         $TableParam2{Border} = 0;
-        $TableParam2{Font} = 'Courier';
+        $TableParam2{Font} = 'Monospaced';
         $TableParam2{FontSize} = 7;
         $TableParam2{BackgroundColor} = '#DDDDDD';
         $TableParam2{Padding} = 4;
@@ -886,16 +934,35 @@ sub _PDFOutputArticles {
             }
         }
     }
+    return 1;
 }
 
 sub _HTMLMask {
     my $Self = shift;
     my %Param = @_;
+
+    # output type, if feature is enabled
+    if ($Self->{ConfigObject}->Get('Ticket::Type')) {
+        $Self->{LayoutObject}->Block(
+            Name => 'TicketType',
+            Data => {
+                %Param,
+            },
+        );
+    }
+    # output service and sla, if feature is enabled
+    if ($Self->{ConfigObject}->Get('Ticket::Service')) {
+        $Self->{LayoutObject}->Block(
+            Name => 'TicketService',
+            Data => {
+                %Param,
+            },
+        );
+    }
     # build article stuff
     my $SelectedArticleID = $Param{ArticleID} || '';
     my @ArticleBox = @{$Param{ArticleBox}};
     # get last customer article
-    my $Output = '';
     foreach my $ArticleTmp (@ArticleBox) {
         my %Article = %{$ArticleTmp};
         # get attacment string
@@ -921,7 +988,7 @@ sub _HTMLMask {
         else {
             # html quoting
             $Article{Body} = $Self->{LayoutObject}->Ascii2Html(
-                NewLine => $Self->{ConfigObject}->Get('DefaultViewNewLine') || 90,
+                NewLine => $Self->{ConfigObject}->Get('DefaultViewNewLine'),
                 Text => $Article{Body},
                 VMax => $Self->{ConfigObject}->Get('DefaultViewLines') || 5000,
             );
