@@ -2,7 +2,7 @@
 # Kernel/Modules/AgentTicketResponsible.pm - set ticket responsible
 # Copyright (C) 2001-2007 OTRS GmbH, http://otrs.org/
 # --
-# $Id: AgentTicketResponsible.pm,v 1.10.2.1 2007/02/26 11:12:28 martin Exp $
+# $Id: AgentTicketResponsible.pm,v 1.21 2007/06/18 09:33:57 martin Exp $
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
 # the enclosed file COPYING for license information (GPL). If you
@@ -16,7 +16,7 @@ use Kernel::System::State;
 use Kernel::System::Web::UploadCache;
 
 use vars qw($VERSION);
-$VERSION = '$Revision: 1.10.2.1 $';
+$VERSION = '$Revision: 1.21 $';
 $VERSION =~ s/^\$.*:\W(.*)\W.+?$/$1/;
 
 sub new {
@@ -152,6 +152,7 @@ sub Run {
     foreach (qw(
         NewStateID NewPriorityID TimeUnits ArticleTypeID Title Body Subject
         Year Month Day Hour Minute NewOwnerID NewOwnerType OldOwnerID NewResponsibleID
+        TypeID ServiceID SLAID Expand
         AttachmentUpload
         AttachmentDelete1 AttachmentDelete2 AttachmentDelete3 AttachmentDelete4
         AttachmentDelete5 AttachmentDelete6 AttachmentDelete7 AttachmentDelete8
@@ -162,19 +163,22 @@ sub Run {
     }
     # get ticket free text params
     foreach (1..16) {
-        $GetParam{"TicketFreeKey$_"} =  $Self->{ParamObject}->GetParam(Param => "TicketFreeKey$_");
-        $GetParam{"TicketFreeText$_"} =  $Self->{ParamObject}->GetParam(Param => "TicketFreeText$_");
+        $GetParam{"TicketFreeKey$_"} = $Self->{ParamObject}->GetParam(Param => "TicketFreeKey$_");
+        $GetParam{"TicketFreeText$_"} = $Self->{ParamObject}->GetParam(Param => "TicketFreeText$_");
     }
-    # get ticket free text params
-    foreach (1..2) {
-        foreach my $Type (qw(Year Month Day Hour Minute)) {
-            $GetParam{"TicketFreeTime".$_.$Type} =  $Self->{ParamObject}->GetParam(Param => "TicketFreeTime".$_.$Type);
+    # get ticket free time params
+    foreach (1..6) {
+        foreach my $Type (qw(Used Year Month Day Hour Minute)) {
+            $GetParam{"TicketFreeTime".$_.$Type} = $Self->{ParamObject}->GetParam(Param => "TicketFreeTime".$_.$Type);
+        }
+        if (!$Self->{ConfigObject}->Get('TicketFreeTimeOptional'.$_)) {
+            $GetParam{'TicketFreeTime'.$_.'Used'} = 1;
         }
     }
     # get article free text params
     foreach (1..3) {
-        $GetParam{"ArticleFreeKey$_"} =  $Self->{ParamObject}->GetParam(Param => "ArticleFreeKey$_");
-        $GetParam{"ArticleFreeText$_"} =  $Self->{ParamObject}->GetParam(Param => "ArticleFreeText$_");
+        $GetParam{"ArticleFreeKey$_"} = $Self->{ParamObject}->GetParam(Param => "ArticleFreeKey$_");
+        $GetParam{"ArticleFreeText$_"} = $Self->{ParamObject}->GetParam(Param => "ArticleFreeText$_");
     }
     # rewrap body if exists
     if ($GetParam{Body}) {
@@ -217,6 +221,16 @@ sub Run {
                 $Error{"Body invalid"} = '* invalid';
             }
         }
+        # check required FreeTextField (if configured)
+        foreach (1..16) {
+            if ($Self->{Config}->{'TicketFreeText'}->{$_} == 2 && $GetParam{"TicketFreeText$_"} eq '') {
+                $Error{"TicketFreeTextField$_ invalid"} = '* invalid';
+            }
+        }
+        # check if service is selected
+        if ($Self->{ConfigObject}->Get('Ticket::Service') && $GetParam{SLAID} && !$GetParam{ServiceID}) {
+            $Error{"Service invalid"} = '* invalid';
+        }
         # attachment delete
         foreach (1..16) {
             if ($GetParam{"AttachmentDelete$_"}) {
@@ -243,6 +257,11 @@ sub Run {
         my @Attachments = $Self->{UploadCachObject}->FormIDGetAllFilesMeta(
             FormID => $Self->{FormID},
         );
+        # check expand
+        if ($GetParam{Expand}) {
+            %Error = ();
+            $Error{"Expand"} = 1;
+        }
         # check errors
         if (%Error) {
             # ticket free text
@@ -313,6 +332,31 @@ sub Run {
                 );
             }
         }
+        if ($Self->{ConfigObject}->Get('Ticket::Type') && $Self->{Config}->{TicketType}) {
+            if (defined($GetParam{TypeID})) {
+                $Self->{TicketObject}->TicketTypeSet(
+                    TypeID => $GetParam{TypeID},
+                    TicketID => $Self->{TicketID},
+                    UserID => $Self->{UserID},
+                );
+            }
+        }
+        if ($Self->{ConfigObject}->Get('Ticket::Service') && $Self->{Config}->{Service}) {
+            if (defined($GetParam{ServiceID})) {
+                $Self->{TicketObject}->TicketServiceSet(
+                    ServiceID => $GetParam{ServiceID},
+                    TicketID => $Self->{TicketID},
+                    UserID => $Self->{UserID},
+                );
+            }
+            if (defined($GetParam{SLAID})) {
+                $Self->{TicketObject}->TicketSLASet(
+                    SLAID => $GetParam{SLAID},
+                    TicketID => $Self->{TicketID},
+                    UserID => $Self->{UserID},
+                );
+            }
+        }
         if ($Self->{Config}->{Owner}) {
             if ($GetParam{NewOwnerType} eq 'Old' && $GetParam{OldOwnerID}) {
                 $Self->{TicketObject}->LockSet(
@@ -420,16 +464,27 @@ sub Run {
             }
         }
         # set ticket free time
-        foreach (1..2) {
+        foreach (1..6) {
             if (defined($GetParam{"TicketFreeTime".$_."Year"}) &&
                 defined($GetParam{"TicketFreeTime".$_."Month"}) &&
                 defined($GetParam{"TicketFreeTime".$_."Day"}) &&
                 defined($GetParam{"TicketFreeTime".$_."Hour"}) &&
-                defined($GetParam{"TicketFreeTime".$_."Minute"})) {
-                my %Time = $Self->{LayoutObject}->TransfromDateSelection(
-                    %GetParam,
-                    Prefix => "TicketFreeTime".$_,
-                );
+                defined($GetParam{"TicketFreeTime".$_."Minute"})
+            ) {
+                my %Time;
+                $Time{"TicketFreeTime".$_."Year"} = 0;
+                $Time{"TicketFreeTime".$_."Month"} = 0;
+                $Time{"TicketFreeTime".$_."Day"} = 0;
+                $Time{"TicketFreeTime".$_."Hour"} = 0;
+                $Time{"TicketFreeTime".$_."Minute"} = 0;
+                $Time{"TicketFreeTime".$_."Secunde"} = 0;
+
+                if ($GetParam{"TicketFreeTime".$_."Used"}) {
+                    %Time = $Self->{LayoutObject}->TransfromDateSelection(
+                        %GetParam,
+                        Prefix => "TicketFreeTime".$_,
+                    );
+                }
                 $Self->{TicketObject}->TicketFreeTimeSet(
                     %Time,
                     Prefix => "TicketFreeTime",
@@ -490,7 +545,9 @@ sub Run {
             return $Self->{LayoutObject}->Redirect(OP => $Self->{LastScreenOverview});
         }
         # redirect
-        return $Self->{LayoutObject}->Redirect(OP => $Self->{LastScreenView});
+        return $Self->{LayoutObject}->Redirect(
+            OP => "Action=AgentTicketZoom&TicketID=$Self->{TicketID}&ArticleID=$ArticleID",
+        );
     }
     else {
         # fillup vars
@@ -520,18 +577,28 @@ sub Run {
             Ticket => \%Ticket,
             Config => \%TicketFreeText,
         );
-        # get free text params
+        # free time
         my %TicketFreeTime = ();
-        foreach (1..2) {
+        foreach (1..6) {
+            $TicketFreeTime{"TicketFreeTime".$_.'Optional'} = $Self->{ConfigObject}->Get('TicketFreeTimeOptional'.$_) || 0;
+            $TicketFreeTime{"TicketFreeTime".$_.'Used'} = $GetParam{'TicketFreeTime'.$_.'Used'};
+
             if ($Ticket{"TicketFreeTime".$_}) {
-                ($TicketFreeTime{"TicketFreeTime".$_.'Secunde'}, $TicketFreeTime{"TicketFreeTime".$_.'Minute'}, $TicketFreeTime{"TicketFreeTime".$_.'Hour'}, $TicketFreeTime{"TicketFreeTime".$_.'Day'}, $TicketFreeTime{"TicketFreeTime".$_.'Month'},  $TicketFreeTime{"TicketFreeTime".$_.'Year'}) = $Self->{TimeObject}->SystemTime2Date(
+                (
+                    $TicketFreeTime{"TicketFreeTime".$_.'Secunde'},
+                    $TicketFreeTime{"TicketFreeTime".$_.'Minute'},
+                    $TicketFreeTime{"TicketFreeTime".$_.'Hour'},
+                    $TicketFreeTime{"TicketFreeTime".$_.'Day'},
+                    $TicketFreeTime{"TicketFreeTime".$_.'Month'},
+                    $TicketFreeTime{"TicketFreeTime".$_.'Year'}
+                ) = $Self->{TimeObject}->SystemTime2Date(
                     SystemTime => $Self->{TimeObject}->TimeStamp2SystemTime(
                         String => $Ticket{"TicketFreeTime".$_},
                     ),
                 );
+                $TicketFreeTime{"TicketFreeTime".$_.'Used'} = 1;
             }
         }
-        # free time
         my %TicketFreeTimeHTML = $Self->{LayoutObject}->AgentFreeDate(
             Ticket => \%TicketFreeTime,
         );
@@ -573,6 +640,11 @@ sub Run {
 sub _Mask {
     my $Self = shift;
     my %Param = @_;
+    # get list type
+    my $TreeView = 0;
+    if ($Self->{ConfigObject}->Get('Ticket::Frontend::ListType') eq 'tree') {
+        $TreeView = 1;
+    }
     my %Ticket = $Self->{TicketObject}->TicketGet(TicketID => $Self->{TicketID});
 
     if ($Self->{Config}->{Title}) {
@@ -581,6 +653,74 @@ sub _Mask {
             Data => \%Param,
         );
     }
+    # types
+    if ($Self->{ConfigObject}->Get('Ticket::Type') && $Self->{Config}->{TicketType}) {
+        my %Type = $Self->{TicketObject}->TicketTypeList(
+            %Param,
+            Action => $Self->{Action},
+            UserID => $Self->{UserID},
+        );
+        $Param{'TypeStrg'} = $Self->{LayoutObject}->BuildSelection(
+            Data => \%Type,
+            Name => 'TypeID',
+            SelectedID => $Param{TypeID},
+            PossibleNone => 1,
+            Sort => 'AlphanumericValue',
+            Translation => 0,
+            OnChange => "document.compose.Expand.value='3'; document.compose.submit(); return false;",
+        );
+        $Self->{LayoutObject}->Block(
+            Name => 'Type',
+            Data => {%Param},
+        );
+    }
+    # services
+    if ($Self->{ConfigObject}->Get('Ticket::Service') && $Self->{Config}->{Service}) {
+        my %Service = ('', '-');
+        if ($Ticket{CustomerUserID}) {
+            %Service = $Self->{TicketObject}->TicketServiceList(
+                %Param,
+                Action => $Self->{Action},
+                CustomerUserID => $Ticket{CustomerUserID},
+                UserID => $Self->{UserID},
+            );
+        }
+        $Param{'ServiceStrg'} = $Self->{LayoutObject}->BuildSelection(
+            Data => \%Service,
+            Name => 'ServiceID',
+            SelectedID => $Param{ServiceID},
+            PossibleNone => 1,
+            TreeView => $TreeView,
+            Sort => 'TreeView',
+            Translation => 0,
+            OnChange => "document.compose.Expand.value='3'; document.compose.submit(); return false;",
+        );
+        $Self->{LayoutObject}->Block(
+            Name => 'Service',
+            Data => {%Param},
+        );
+        my %SLA = ();
+        if ($Param{ServiceID}) {
+            %SLA = $Self->{TicketObject}->TicketSLAList(
+                %Param,
+                Action => $Self->{Action},
+                UserID => $Self->{UserID},
+            );
+        }
+        $Param{'SLAStrg'} = $Self->{LayoutObject}->BuildSelection(
+            Data => \%SLA,
+            Name => 'SLAID',
+            SelectedID => $Param{SLAID},
+            PossibleNone => 1,
+            Sort => 'AlphanumericValue',
+            Translation => 0,
+            OnChange => "document.compose.Expand.value='3'; document.compose.submit(); return false;",
+        );
+        $Self->{LayoutObject}->Block(
+            Name => 'SLA',
+            Data => {%Param},
+        );
+    }
     if ($Self->{Config}->{Owner}) {
         # get user of own groups
         my %ShownUsers = ();
@@ -619,7 +759,7 @@ sub _Mask {
                 if ($Counter) {
                     if (!$UserHash{$User->{UserID}}) {
                         $UserHash{$User->{UserID}} = "$Counter: $User->{UserLastname} ".
-                          "$User->{UserFirstname} ($User->{UserLogin})";
+                            "$User->{UserFirstname} ($User->{UserLogin})";
                     }
                 }
                 $Counter++;
@@ -628,10 +768,17 @@ sub _Mask {
         if (!%UserHash) {
             $UserHash{''} = '-';
         }
+        my $OldOwnerSelectedID = '';
+        if ($Param{OldOwnerID}) {
+            $OldOwnerSelectedID = $Param{OldOwnerID};
+        }
+        elsif ($OldUserInfo[0]->{UserID}) {
+            $OldOwnerSelectedID = $OldUserInfo[0]->{UserID} . '1';
+        }
         # build string
         $Param{'OldOwnerStrg'} = $Self->{LayoutObject}->OptionStrgHashRef(
             Data => \%UserHash,
-            SelectedID => $Param{OldOwnerID} || $OldUserInfo[0]->{UserID}.'1',
+            SelectedID => $OldOwnerSelectedID,
             Name => 'OldOwnerID',
             OnClick => "change_selected(2)",
         );
@@ -693,7 +840,6 @@ sub _Mask {
         );
         if (!$Self->{Config}->{StateDefault}) {
             $StateList{''} = '-';
-#            $State{SelectedID} = $Param{StateID};
         }
         if (!$Param{NewStateID}) {
             if ($Self->{Config}->{StateDefault}) {
@@ -742,7 +888,6 @@ sub _Mask {
         );
         if (!$Self->{Config}->{PriorityDefault}) {
             $PriorityList{''} = '-';
-#            $Priority{SelectedID} = $Param{PriorityID};
         }
         if (!$Param{NewPriorityID}) {
             if ($Self->{Config}->{PriorityDefault}) {
@@ -890,6 +1035,7 @@ sub _Mask {
                     TicketFreeKeyField => $Param{'TicketFreeKeyField'.$Count},
                     TicketFreeTextField => $Param{'TicketFreeTextField'.$Count},
                     Count => $Count,
+                    %Param,
                 },
             );
             $Self->{LayoutObject}->Block(
@@ -902,7 +1048,7 @@ sub _Mask {
         }
     }
     $Count = 0;
-    foreach (1..2) {
+    foreach (1..6) {
         $Count++;
         if ($Self->{Config}->{'TicketFreeTime'}->{$Count}) {
             $Self->{LayoutObject}->Block(
@@ -944,6 +1090,18 @@ sub _Mask {
             );
         }
     }
+    # jscript check freetextfields by submit
+    foreach my $Key (keys %{$Self->{Config}->{TicketFreeText}}) {
+        if ($Self->{Config}->{TicketFreeText}->{$Key} == 2) {
+            $Self->{LayoutObject}->Block(
+                Name => 'TicketFreeTextCheckJs',
+                Data => {
+                    TicketFreeTextField => "TicketFreeText$Key",
+                    TicketFreeKeyField => "TicketFreeKey$Key",
+                },
+            );
+        }
+    }
     # get output back
     return $Self->{LayoutObject}->Output(TemplateFile => 'AgentTicketResponsible', Data => \%Param);
 }
