@@ -1,8 +1,8 @@
 # --
 # Kernel/Modules/AgentTicketSearch.pm - Utilities for tickets
-# Copyright (C) 2001-2006 OTRS GmbH, http://otrs.org/
+# Copyright (C) 2001-2007 OTRS GmbH, http://otrs.org/
 # --
-# $Id: AgentTicketSearch.pm,v 1.33 2006/12/13 08:43:10 martin Exp $
+# $Id: AgentTicketSearch.pm,v 1.42 2007/07/31 11:45:54 martin Exp $
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
 # the enclosed file COPYING for license information (GPL). If you
@@ -14,12 +14,15 @@ package Kernel::Modules::AgentTicketSearch;
 use strict;
 use Kernel::System::CustomerUser;
 use Kernel::System::Priority;
-use Kernel::System::State;
 use Kernel::System::SearchProfile;
+use Kernel::System::Service;
+use Kernel::System::SLA;
+use Kernel::System::State;
+use Kernel::System::Type;
 use Kernel::System::PDF;
 
 use vars qw($VERSION);
-$VERSION = '$Revision: 1.33 $';
+$VERSION = '$Revision: 1.42 $';
 $VERSION =~ s/^\$.*:\W(.*)\W.+?$/$1/;
 
 sub new {
@@ -45,15 +48,19 @@ sub new {
     $Self->{StateObject} = Kernel::System::State->new(%Param);
     $Self->{SearchProfileObject} = Kernel::System::SearchProfile->new(%Param);
     $Self->{PDFObject} = Kernel::System::PDF->new(%Param);
+    $Self->{ServiceObject} = Kernel::System::Service->new(%Param);
+    $Self->{SLAObject} = Kernel::System::SLA->new(%Param);
+    $Self->{TypeObject} = Kernel::System::Type->new(%Param);
 
     # if we need to do a fulltext search on an external mirror database
-    if ($Self->{ConfigObject}->Get('Ticket::Frontend::Search::DB::DSN')) {
+    if ($Self->{ConfigObject}->Get('Core::MirrorDB::DSN')) {
         my $ExtraDatabaseObject = Kernel::System::DB->new(
             LogObject => $Param{LogObject},
             ConfigObject => $Param{ConfigObject},
-            DatabaseDSN => $Self->{ConfigObject}->Get('Ticket::Frontend::Search::DB::DSN'),
-            DatabaseUser => $Self->{ConfigObject}->Get('Ticket::Frontend::Search::DB::User'),
-            DatabasePw => $Self->{ConfigObject}->Get('Ticket::Frontend::Search::DB::Password'),
+            MainObject => $Param{MainObject},
+            DatabaseDSN => $Self->{ConfigObject}->Get('Core::MirrorDB::DSN'),
+            DatabaseUser => $Self->{ConfigObject}->Get('Core::MirrorDB::User'),
+            DatabasePw => $Self->{ConfigObject}->Get('Core::MirrorDB::Password'),
         );
         if (!$ExtraDatabaseObject) {
             $Self->{LayoutObject}->FatalError();
@@ -89,7 +96,9 @@ sub Run {
     $Self->{EraseTemplate} = $Self->{ParamObject}->GetParam(Param => 'EraseTemplate') || '';
     # check request
     if ($Self->{ParamObject}->GetParam(Param => 'SearchTemplate') && $Self->{Profile}) {
-        return $Self->{LayoutObject}->Redirect(OP => "Action=AgentTicketSearch&Subaction=Search&TakeLastSearch=1&SaveProfile=1&Profile=$Self->{Profile}");
+        return $Self->{LayoutObject}->Redirect(
+            OP => "Action=AgentTicketSearch&Subaction=Search&TakeLastSearch=1&SaveProfile=1&Profile=$Self->{Profile}"
+        );
     }
     # get signle params
     my %GetParam = ();
@@ -116,6 +125,36 @@ sub Run {
             TicketFreeTime2StartYear
             TicketFreeTime2Stop TicketFreeTime2StopDay TicketFreeTime2StopMonth
             TicketFreeTime2StopYear
+            TicketFreeTime3
+            TicketFreeTime3Start TicketFreeTime3StartDay TicketFreeTime3StartMonth
+            TicketFreeTime3StartYear
+            TicketFreeTime3Stop TicketFreeTime3StopDay TicketFreeTime3StopMonth
+            TicketFreeTime3StopYear
+            TicketFreeTime4
+            TicketFreeTime4Start TicketFreeTime4StartDay TicketFreeTime4StartMonth
+            TicketFreeTime4StartYear
+            TicketFreeTime4Stop TicketFreeTime4StopDay TicketFreeTime4StopMonth
+            TicketFreeTime4StopYear
+            TicketFreeTime5
+            TicketFreeTime5Start TicketFreeTime5StartDay TicketFreeTime5StartMonth
+            TicketFreeTime5StartYear
+            TicketFreeTime5Stop TicketFreeTime5StopDay TicketFreeTime5StopMonth
+            TicketFreeTime5StopYear
+            TicketFreeTime6
+            TicketFreeTime6Start TicketFreeTime6StartDay TicketFreeTime6StartMonth
+            TicketFreeTime6StartYear
+            TicketFreeTime6Stop TicketFreeTime6StopDay TicketFreeTime6StopMonth
+            TicketFreeTime6StopYear
+            TicketFreeTime7
+            TicketFreeTime7Start TicketFreeTime7StartDay TicketFreeTime7StartMonth
+            TicketFreeTime7StartYear
+            TicketFreeTime7Stop TicketFreeTime7StopDay TicketFreeTime7StopMonth
+            TicketFreeTime7StopYear
+            TicketFreeTime8
+            TicketFreeTime8Start TicketFreeTime8StartDay TicketFreeTime8StartMonth
+            TicketFreeTime8StartYear
+            TicketFreeTime8Stop TicketFreeTime8StopDay TicketFreeTime8StopMonth
+            TicketFreeTime8StopYear
             TicketCreateTimePointFormat TicketCreateTimePoint
             TicketCreateTimePointStart
             TicketCreateTimeStart TicketCreateTimeStartDay TicketCreateTimeStartMonth
@@ -133,7 +172,8 @@ sub Run {
         }
         # get array params
         foreach (qw(StateIDs StateTypeIDs QueueIDs PriorityIDs OwnerIDs
-            CreatedQueueIDs CreatedUserIDs
+            CreatedQueueIDs CreatedUserIDs WatchUserIDs ResponsibleIDs
+            TypeIDs ServiceIDs SLAIDs
             TicketFreeKey1 TicketFreeText1 TicketFreeKey2 TicketFreeText2
             TicketFreeKey3 TicketFreeText3 TicketFreeKey4 TicketFreeText4
             TicketFreeKey5 TicketFreeText5 TicketFreeKey6 TicketFreeText6
@@ -174,7 +214,8 @@ sub Run {
             $Self->{Profile} = 'last-search';
         }
         # store last queue screen
-        my $URL = "Action=AgentTicketSearch&Subaction=Search&Profile=$Self->{Profile}&SortBy=$Self->{SortBy}&Order=$Self->{Order}&TakeLastSearch=1&StartHit=$Self->{StartHit}";
+        my $URL = "Action=AgentTicketSearch&Subaction=Search&Profile=$Self->{Profile}&SortBy=$Self->{SortBy}".
+            "&Order=$Self->{Order}&TakeLastSearch=1&StartHit=$Self->{StartHit}";
         $Self->{SessionObject}->UpdateSessionID(
             SessionID => $Self->{SessionID},
             Key => 'LastScreenOverview',
@@ -220,13 +261,19 @@ sub Run {
             foreach (qw(Month Day)) {
                 $GetParam{"TicketCreateTimeStop$_"} = sprintf("%02d", $GetParam{"TicketCreateTimeStop$_"});
             }
-            if ($GetParam{TicketCreateTimeStartDay} && $GetParam{TicketCreateTimeStartMonth} && $GetParam{TicketCreateTimeStartYear}) {
+            if ($GetParam{TicketCreateTimeStartDay} &&
+                $GetParam{TicketCreateTimeStartMonth} &&
+                $GetParam{TicketCreateTimeStartYear}
+            ) {
                 $GetParam{TicketCreateTimeNewerDate} = $GetParam{TicketCreateTimeStartYear}.
                     '-'.$GetParam{TicketCreateTimeStartMonth}.
                     '-'.$GetParam{TicketCreateTimeStartDay}.
                     ' 00:00:01';
             }
-            if ($GetParam{TicketCreateTimeStopDay} && $GetParam{TicketCreateTimeStopMonth} && $GetParam{TicketCreateTimeStopYear}) {
+            if ($GetParam{TicketCreateTimeStopDay} &&
+                $GetParam{TicketCreateTimeStopMonth} &&
+                $GetParam{TicketCreateTimeStopYear}
+            ) {
                 $GetParam{TicketCreateTimeOlderDate} = $GetParam{TicketCreateTimeStopYear}.
                     '-'.$GetParam{TicketCreateTimeStopMonth}.
                     '-'.$GetParam{TicketCreateTimeStopDay}.
@@ -234,7 +281,10 @@ sub Run {
             }
         }
         elsif ($GetParam{TimeSearchType} eq 'TimePoint') {
-            if ($GetParam{TicketCreateTimePoint} && $GetParam{TicketCreateTimePointStart} && $GetParam{TicketCreateTimePointFormat}) {
+            if ($GetParam{TicketCreateTimePoint} &&
+                $GetParam{TicketCreateTimePointStart} &&
+                $GetParam{TicketCreateTimePointFormat}
+            ) {
                 my $Time = 0;
                 if ($GetParam{TicketCreateTimePointFormat} eq 'minute') {
                     $Time = $GetParam{TicketCreateTimePoint};
@@ -263,7 +313,7 @@ sub Run {
             }
         }
         # free time
-        foreach (1..2) {
+        foreach (1..6) {
             if (!$GetParam{'TicketFreeTime'.$_}) {
                 foreach my $Type (qw(Year Month Day)) {
                     $GetParam{'TicketFreeTime'.$_.'Start'.$Type} = undef;
@@ -308,6 +358,7 @@ sub Run {
             OrderBy => $Self->{Order},
             Limit => $Self->{SearchLimit},
             UserID => $Self->{UserID},
+            ConditionInline => $Self->{ConfigObject}->Get('ExtendedSearchCondition'),
             %GetParam,
         );
 
@@ -327,7 +378,8 @@ sub Run {
                     );
                     foreach my $Articles (@Article) {
                         if ($Articles->{Body}) {
-                            $Data{ArticleTree} .= "\n-->||$Articles->{ArticleType}||$Articles->{From}||".$Articles->{Created}."||<--------------\n".$Articles->{Body};
+                            $Data{ArticleTree} .= "\n-->||$Articles->{ArticleType}||$Articles->{From}||".
+                                $Articles->{Created}."||<--------------\n".$Articles->{Body};
                         }
                     }
                 }
@@ -434,8 +486,10 @@ sub Run {
                 elsif ($GetParam{ResultForm} eq 'Print') {
                     if ($Self->{PDFObject}) {
                         my %Info = (%Data, %UserInfo),
-
-                        my $Created = $Self->{LayoutObject}->Output(Template => '$TimeLong{"$Data{"Created"}"}', Data => \%Data);
+                        my $Created = $Self->{LayoutObject}->Output(
+                            Template => '$TimeLong{"$Data{"Created"}"}',
+                            Data => \%Data,
+                        );
                         my $Owner = $Self->{LayoutObject}->Output(
                             Template => '$QData{"Owner","30"} ($Quote{"$Data{"UserFirstname"} $Data{"UserLastname"}","30"})',
                             Data => \%Info
@@ -545,21 +599,21 @@ sub Run {
                 # create the header
                 my $CellData;
                 $CellData->[0]->[0]->{Content} = $Self->{ConfigObject}->Get('Ticket::Hook');
-                $CellData->[0]->[0]->{Font} = 'HelveticaBold';
+                $CellData->[0]->[0]->{Font} = 'ProportionalBold';
                 $CellData->[0]->[1]->{Content} = $Self->{LayoutObject}->{LanguageObject}->Get('Created');
-                $CellData->[0]->[1]->{Font} = 'HelveticaBold';
+                $CellData->[0]->[1]->{Font} = 'ProportionalBold';
                 $CellData->[0]->[2]->{Content} = $Self->{LayoutObject}->{LanguageObject}->Get('From');
-                $CellData->[0]->[2]->{Font} = 'HelveticaBold';
+                $CellData->[0]->[2]->{Font} = 'ProportionalBold';
                 $CellData->[0]->[3]->{Content} = $Self->{LayoutObject}->{LanguageObject}->Get('Subject');
-                $CellData->[0]->[3]->{Font} = 'HelveticaBold';
+                $CellData->[0]->[3]->{Font} = 'ProportionalBold';
                 $CellData->[0]->[4]->{Content} = $Self->{LayoutObject}->{LanguageObject}->Get('State');
-                $CellData->[0]->[4]->{Font} = 'HelveticaBold';
+                $CellData->[0]->[4]->{Font} = 'ProportionalBold';
                 $CellData->[0]->[5]->{Content} = $Self->{LayoutObject}->{LanguageObject}->Get('Queue');
-                $CellData->[0]->[5]->{Font} = 'HelveticaBold';
+                $CellData->[0]->[5]->{Font} = 'ProportionalBold';
                 $CellData->[0]->[6]->{Content} = $Self->{LayoutObject}->{LanguageObject}->Get('Owner');
-                $CellData->[0]->[6]->{Font} = 'HelveticaBold';
+                $CellData->[0]->[6]->{Font} = 'ProportionalBold';
                 $CellData->[0]->[7]->{Content} = $Self->{LayoutObject}->{LanguageObject}->Get('CustomerID');
-                $CellData->[0]->[7]->{Font} = 'HelveticaBold';
+                $CellData->[0]->[7]->{Font} = 'ProportionalBold';
                 # create the content array
                 my $CounterRow = 1;
                 foreach my $Row (@PDFData) {
@@ -604,6 +658,7 @@ sub Run {
                 # create new pdf document
                 $Self->{PDFObject}->DocumentNew(
                     Title => $Self->{ConfigObject}->Get('Product') . ': ' . $Title,
+                    Encode => $Self->{LayoutObject}->{UserCharset},
                 );
                 # start table output
                 $Self->{PDFObject}->PageNew(
@@ -701,10 +756,6 @@ sub Run {
             %GetParam = ();
             $Self->{Profile} = '';
         }
-        # set profile to zero
-#        elsif (!$Self->{SelectTemplate}) {
-#            $Self->{Profile} = '';
-#        }
         # generate search mask
         my $Output = $Self->{LayoutObject}->Header();
         my %LockedData = $Self->{TicketObject}->GetLockedCount(UserID => $Self->{UserID});
@@ -829,7 +880,7 @@ sub MaskForm {
         SelectedIDRefArray => $Param{CreatedQueueIDs},
         OnChangeSubmit => 0,
     );
-    $Param{'PriotitiesStrg'} = $Self->{LayoutObject}->OptionStrgHashRef(
+    $Param{'PrioritiesStrg'} = $Self->{LayoutObject}->OptionStrgHashRef(
         Data => {
             $Self->{PriorityObject}->PriorityList(
                 UserID => $Self->{UserID},
@@ -937,7 +988,7 @@ sub MaskForm {
         Prefix => 'TicketCreateTimeStop',
         Format => 'DateInputFormat',
     );
-    foreach (1..2) {
+    foreach (1..6) {
         $Param{'TicketFreeTime'.$_.'Start'} = $Self->{LayoutObject}->BuildDateSelection(
             %Param,
             Prefix => 'TicketFreeTime'.$_.'Start',
@@ -958,6 +1009,131 @@ sub MaskForm {
             %Param,
         },
     );
+    # build type string
+    if ($Self->{ConfigObject}->Get('Ticket::Type')) {
+        my %Type = $Self->{TypeObject}->TypeList(
+            UserID => $Self->{UserID},
+        );
+        $Param{'TypesStrg'} = $Self->{LayoutObject}->BuildSelection(
+            Data => \%Type,
+            Name => 'TypeIDs',
+            SelectedID => $Param{TypeIDs},
+            Sort => 'AlphanumericValue',
+            Size => 3,
+            Multiple => 1,
+            Translation => 0,
+        );
+        $Self->{LayoutObject}->Block(
+            Name => 'TicketType',
+            Data => {%Param},
+        );
+    }
+    # build service string
+    if ($Self->{ConfigObject}->Get('Ticket::Service')) {
+        # get list type
+        my $TreeView = 0;
+        if ($Self->{ConfigObject}->Get('Ticket::Frontend::ListType') eq 'tree') {
+            $TreeView = 1;
+        }
+        my %Service = $Self->{ServiceObject}->ServiceList(
+            UserID => $Self->{UserID},
+        );
+        $Param{'ServicesStrg'} = $Self->{LayoutObject}->BuildSelection(
+            Data => \%Service,
+            Name => 'ServiceIDs',
+            SelectedID => $Param{ServiceIDs},
+            TreeView => $TreeView,
+            Sort => 'TreeView',
+            Size => 5,
+            Multiple => 1,
+            Translation => 0,
+        );
+        my %SLA = $Self->{SLAObject}->SLAList(
+            UserID => $Self->{UserID},
+        );
+        $Param{'SLAsStrg'} = $Self->{LayoutObject}->BuildSelection(
+            Data => \%SLA,
+            Name => 'SLAIDs',
+            SelectedID => $Param{SLAIDs},
+            Sort => 'AlphanumericValue',
+            Size => 5,
+            Multiple => 1,
+            Translation => 0,
+        );
+        $Self->{LayoutObject}->Block(
+            Name => 'TicketService',
+            Data => {%Param},
+        );
+    }
+    if ($Self->{ConfigObject}->Get('Ticket::Watcher') || $Self->{ConfigObject}->Get('Ticket::Responsible')) {
+        $Self->{LayoutObject}->Block(
+            Name => 'TicketResponsibleWatcher',
+        );
+        if ($Self->{ConfigObject}->Get('Ticket::Watcher')) {
+            $Self->{LayoutObject}->Block(
+                Name => 'TicketResponsibleWatcherHeaderOn',
+                Data => {
+                    Headline => 'Watcher',
+                },
+            );
+            my $SelectStrg = $Self->{LayoutObject}->BuildSelection(
+                Data => \%ShownUsers,
+                Name => 'WatchUserIDs',
+                SelectedID => $Param{WatchUserIDs},
+                Sort => 'AlphanumericValue',
+                Size => 5,
+                Multiple => 1,
+                Translation => 0,
+            );
+            $Self->{LayoutObject}->Block(
+                Name => 'TicketResponsibleWatcherBodyOn',
+                Data => {
+                    %Param,
+                    SelectStrg => $SelectStrg,
+                },
+            );
+        }
+        else {
+            $Self->{LayoutObject}->Block(
+                Name => 'TicketResponsibleWatcherHeaderOff',
+            );
+            $Self->{LayoutObject}->Block(
+                Name => 'TicketResponsibleWatcherBodyOff',
+            );
+        }
+        if ($Self->{ConfigObject}->Get('Ticket::Responsible')) {
+            $Self->{LayoutObject}->Block(
+                Name => 'TicketResponsibleWatcherHeaderOn',
+                Data => {
+                    Headline => 'Responsible',
+                },
+            );
+            my $SelectStrg = $Self->{LayoutObject}->BuildSelection(
+                Data => \%ShownUsers,
+                Name => 'ResponsibleIDs',
+                SelectedID => $Param{ResponsibleIDs},
+                Sort => 'AlphanumericValue',
+                Size => 5,
+                Multiple => 1,
+                Translation => 0,
+            );
+            $Self->{LayoutObject}->Block(
+                Name => 'TicketResponsibleWatcherBodyOn',
+                Data => {
+                    %Param,
+                    SelectStrg => $SelectStrg,
+                },
+            );
+        }
+        else {
+            $Self->{LayoutObject}->Block(
+                Name => 'TicketResponsibleWatcherHeaderOff',
+            );
+            $Self->{LayoutObject}->Block(
+                Name => 'TicketResponsibleWatcherBodyOff',
+            );
+        }
+    }
     my $Count = 0;
     foreach (1..16) {
         $Count++;
@@ -979,7 +1155,7 @@ sub MaskForm {
         }
     }
     $Count = 0;
-    foreach (1..2) {
+    foreach (1..6) {
         $Count++;
         if ($Self->{Config}->{'TicketFreeTime'}->{$Count}) {
             $Self->{LayoutObject}->Block(
