@@ -2,7 +2,7 @@
 # Kernel/System/CustomerUser/DB.pm - some customer user functions
 # Copyright (C) 2001-2007 OTRS GmbH, http://otrs.org/
 # --
-# $Id: DB.pm,v 1.42.2.1 2007/01/08 21:14:04 martin Exp $
+# $Id: DB.pm,v 1.51 2007/07/26 14:13:33 martin Exp $
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
 # the enclosed file COPYING for license information (GPL). If you
@@ -12,11 +12,15 @@
 package Kernel::System::CustomerUser::DB;
 
 use strict;
+use warnings;
 use Kernel::System::CheckItem;
+use Kernel::System::Valid;
+use Kernel::System::Cache;
+
 use Crypt::PasswdMD5 qw(unix_md5_crypt);
 
 use vars qw(@ISA $VERSION);
-$VERSION = '$Revision: 1.42.2.1 $';
+$VERSION = '$Revision: 1.51 $';
 $VERSION =~ s/^\$.*:\W(.*)\W.+?$/$1/;
 
 sub new {
@@ -30,6 +34,15 @@ sub new {
     foreach (qw(DBObject ConfigObject LogObject PreferencesObject CustomerUserMap)) {
         $Self->{$_} = $Param{$_} || die "Got no $_!";
     }
+    # create valid object
+    $Self->{ValidObject} = Kernel::System::Valid->new(%Param);
+    # create check item object
+    $Self->{CheckItemObject} = Kernel::System::CheckItem->new(%Param);
+    # create cache object
+    if ($Self->{CustomerUserMap}->{'CacheTTL'}) {
+        $Self->{CacheObject} = Kernel::System::Cache->new(%Param);
+    }
+
     # max shown user a search list
     $Self->{UserSearchListLimit} = $Self->{CustomerUserMap}->{'CustomerUserSearchListLimit'} || 250;
     # config options
@@ -64,8 +77,7 @@ sub new {
         # remember that we have the DBObject not from parent call
         $Self->{NotParentDBObject} = 1;
     }
-    # create check item object
-    $Self->{CheckItemObject} = Kernel::System::CheckItem->new(%Param);
+
     return $Self;
 }
 
@@ -90,6 +102,15 @@ sub CustomerName {
     }
     $SQL .= " FROM $Self->{CustomerTable} WHERE ".
         " LOWER($Self->{CustomerKey}) = LOWER('".$Self->{DBObject}->Quote($Param{UserLogin})."')";
+    # check cache
+    if ($Self->{CacheObject}) {
+        my $Name = $Self->{CacheObject}->Get(
+            Key => "CustomerUser::CustomerName::$SQL",
+        );
+        if (defined($Name)) {
+            return $Name;
+        }
+    }
     # get data
     $Self->{DBObject}->Prepare(SQL => $SQL, Limit => 1);
     while (my @Row = $Self->{DBObject}->FetchrowArray()) {
@@ -104,6 +125,14 @@ sub CustomerName {
             }
         }
     }
+    # cache request
+    if ($Self->{CacheObject}) {
+        $Self->{CacheObject}->Set(
+            Key => "CustomerUser::CustomerName::$SQL",
+            Value => $Name,
+            TTL => $Self->{CustomerUserMap}->{'CacheTTL'},
+        );
+    }
     return $Name;
 }
 
@@ -178,20 +207,37 @@ sub CustomerSearch {
     # add valid option
     if ($Self->{CustomerUserMap}->{CustomerValid} && $Valid) {
         $SQL .= "AND ".$Self->{CustomerUserMap}->{CustomerValid}.
-        " IN ( ${\(join ', ', $Self->{DBObject}->GetValidIDs())} ) ";
+        " IN ( ${\(join ', ', $Self->{ValidObject}->ValidIDsGet())} ) ";
+    }
+    # check cache
+    if ($Self->{CacheObject}) {
+        my $Users = $Self->{CacheObject}->Get(
+            Key => "CustomerUser::CustomerSearch::$SQL",
+        );
+        if ($Users) {
+            return %{$Users};
+        }
     }
     # get data
     $Self->{DBObject}->Prepare(SQL => $SQL, Limit => $Self->{UserSearchListLimit});
     while (my @Row = $Self->{DBObject}->FetchrowArray()) {
         if (!$Users{$Row[0]}) {
-             foreach (1..8) {
-                 if ($Row[$_]) {
-                      $Users{$Row[0]} .= $Row[$_].' ';
-                 }
-             }
-             $Users{$Row[0]} =~ s/^(.*)\s(.+?\@.+?\..+?)(\s|)$/"$1" <$2>/;
+            foreach (1..8) {
+                if ($Row[$_]) {
+                    $Users{$Row[0]} .= $Row[$_].' ';
+                }
+            }
+            $Users{$Row[0]} =~ s/^(.*)\s(.+?\@.+?\..+?)(\s|)$/"$1" <$2>/;
         }
     }
+    # cache request
+    if ($Self->{CacheObject}) {
+        $Self->{CacheObject}->Set(
+            Key => "CustomerUser::CustomerSearch::$SQL",
+            Value => \%Users,
+            TTL => $Self->{CustomerUserMap}->{'CacheTTL'},
+        );
+    }
     return %Users;
 }
 
@@ -199,6 +245,15 @@ sub CustomerUserList {
     my $Self = shift;
     my %Param = @_;
     my $Valid = defined $Param{Valid} ? $Param{Valid} : 1;
+    # check cache
+    if ($Self->{CacheObject}) {
+        my $Users = $Self->{CacheObject}->Get(
+            Key => "CustomerUser::CustomerUserList::$Valid",
+        );
+        if ($Users) {
+            return %{$Users};
+        }
+    }
     # get data
     my %Users = $Self->{DBObject}->GetTableData(
         What => "$Self->{CustomerKey}, $Self->{CustomerKey}, $Self->{CustomerID}",
@@ -206,6 +261,14 @@ sub CustomerUserList {
         Clamp => 1,
         Valid => $Valid,
     );
+    # cache request
+    if ($Self->{CacheObject}) {
+        $Self->{CacheObject}->Set(
+            Key => "CustomerUser::CustomerUserList::$Valid",
+            Value => \%Users,
+            TTL => $Self->{CustomerUserMap}->{'CacheTTL'},
+        );
+    }
     return %Users;
 }
 
@@ -218,6 +281,15 @@ sub CustomerIDs {
         $Self->{LogObject}->Log(Priority => 'error', Message => "Need User!");
         return;
     }
+    # check cache
+    if ($Self->{CacheObject}) {
+        my $CustomerIDs = $Self->{CacheObject}->Get(
+            Key => "CustomerUser::CustomerIDs::$Param{User}",
+        );
+        if ($CustomerIDs) {
+            return @{$CustomerIDs};
+        }
+    }
     # get customer data
     my %Data = $Self->CustomerUserDataGet(
         User => $Param{User},
@@ -244,6 +316,14 @@ sub CustomerIDs {
     if ($Data{UserCustomerID} && !$Self->{ExcludePrimaryCustomerID}) {
         push (@CustomerIDs, $Data{UserCustomerID});
     }
+    # cache request
+    if ($Self->{CacheObject}) {
+        $Self->{CacheObject}->Set(
+            Key => "CustomerUser::CustomerIDs::$Param{User}",
+            Value => \@CustomerIDs,
+            TTL => $Self->{CustomerUserMap}->{'CacheTTL'},
+        );
+    }
     return @CustomerIDs;
 }
 
@@ -263,12 +343,30 @@ sub CustomerUserDataGet {
     }
     $SQL .= $Self->{CustomerKey}." FROM $Self->{CustomerTable} WHERE ";
     if ($Param{User}) {
+        # check cache
+        if ($Self->{CacheObject}) {
+            my $Data = $Self->{CacheObject}->Get(
+                Key => "CustomerUser::CustomerUserDataGet::User::$Param{User}",
+            );
+            if ($Data) {
+                return %{$Data};
+            }
+        }
         $SQL .= "LOWER($Self->{CustomerKey}) = LOWER('".$Self->{DBObject}->Quote($Param{User})."')";
     }
     elsif ($Param{CustomerID}) {
+        # check cache
+        if ($Self->{CacheObject}) {
+            my $Data = $Self->{CacheObject}->Get(
+                Key => "CustomerUser::CustomerUserDataGet::CustomerID::$Param{CustomerID}",
+            );
+            if ($Data) {
+                return %{$Data};
+            }
+        }
         $SQL .= "LOWER($Self->{CustomerID}) = LOWER('".$Self->{DBObject}->Quote($Param{CustomerID})."')";
     }
-    # get inital data
+    # get initial data
     $Self->{DBObject}->Prepare(SQL => $SQL);
     while (my @Row = $Self->{DBObject}->FetchrowArray()) {
         my $MapCounter = 0;
@@ -279,24 +377,48 @@ sub CustomerUserDataGet {
     }
     # check data
     if (! exists $Data{UserLogin} && $Param{User}) {
-#        $Self->{LogObject}->Log(
-#          Priority => 'notice',
-#          Message => "Panic! No UserData for customer user: '$Param{User}'!!!",
-#        );
+        # cache request
+        if ($Self->{CacheObject}) {
+            $Self->{CacheObject}->Set(
+                Key => "CustomerUser::CustomerUserDataGet::User::$Param{User}",
+                Value => { },
+                TTL => $Self->{CustomerUserMap}->{'CacheTTL'},
+            );
+        }
         return;
     }
     if (! exists $Data{UserLogin} && $Param{CustomerID}) {
-#        $Self->{LogObject}->Log(
-#          Priority => 'notice',
-#          Message => "Panic! No UserData for customer id: '$Param{CustomerID}'!!!",
-#        );
+        # cache request
+        if ($Self->{CacheObject}) {
+            $Self->{CacheObject}->Set(
+                Key => "CustomerUser::CustomerUserDataGet::CustomerID::$Param{CustomerID}",
+                Value => { },
+                TTL => $Self->{CustomerUserMap}->{'CacheTTL'},
+            );
+        }
         return;
     }
     # compat!
     $Data{UserID} = $Data{UserLogin};
     # get preferences
     my %Preferences = $Self->{PreferencesObject}->GetPreferences(UserID => $Data{UserID});
-
+    # cache request
+    if ($Self->{CacheObject}) {
+        if ($Param{User}) {
+            $Self->{CacheObject}->Set(
+                Key => "CustomerUser::CustomerUserDataGet::User::$Param{User}",
+                Value => { %Data, %Preferences },
+                TTL => $Self->{CustomerUserMap}->{'CacheTTL'},
+            );
+        }
+        elsif ($Param{CustomerID}) {
+            $Self->{CacheObject}->Set(
+                Key => "CustomerUser::CustomerUserDataGet::CustomerID::$Param{CustomerID}",
+                Value => { %Data, %Preferences },
+                TTL => $Self->{CustomerUserMap}->{'CacheTTL'},
+            );
+        }
+    }
     # return data
     return (%Data, %Preferences);
 }
@@ -354,37 +476,64 @@ sub CustomerUserAdd {
         $Self->{LogObject}->Log(
             Priority => 'error',
             Message => "Email address ($Param{UserEmail}) not valid (".
-              $Self->{CheckItemObject}->CheckError().")!",
+                $Self->{CheckItemObject}->CheckError().")!",
         );
         return;
     }
+
+    # quote values
+    my %Value;
+    for my $Entry (@{$Self->{CustomerUserMap}->{Map}}) {
+        if ($Entry->[5] =~ /^int$/i) {
+            if ($Param{ $Entry->[0] }) {
+                $Value{ $Entry->[0] } = $Self->{DBObject}->Quote($Param{ $Entry->[0] }, 'Integer');
+            }
+            else {
+                $Value{ $Entry->[0] } = 0;
+            }
+        }
+        else {
+            if ($Param{ $Entry->[0] }) {
+                $Value{ $Entry->[0] } = "'" . $Self->{DBObject}->Quote($Param{$Entry->[0]}) . "'";
+            }
+            else {
+                $Value{ $Entry->[0] }= "''";
+            }
+        }
+    }
+
     # build insert
     my $SQL = "INSERT INTO $Self->{CustomerTable} (";
     foreach my $Entry (@{$Self->{CustomerUserMap}->{Map}}) {
-        $SQL .= " $Entry->[2], ";
+        if ($Entry->[0] !~ /^UserPassword$/i) {
+            $SQL .= " $Entry->[2], ";
+        }
     }
     $SQL .= "create_time, create_by, change_time, change_by)";
     $SQL .= " VALUES (";
     foreach my $Entry (@{$Self->{CustomerUserMap}->{Map}}) {
-        if ($Entry->[5] =~ /^int$/i) {
-            $SQL .= " ".$Self->{DBObject}->Quote($Param{$Entry->[0]}).", ";
-        }
-        else {
-            $SQL .= " '".$Self->{DBObject}->Quote($Param{$Entry->[0]})."', ";
+        if ($Entry->[0] !~ /^UserPassword$/i) {
+            $SQL .= " $Value{ $Entry->[0] }, ";
         }
     }
     $SQL .= "current_timestamp, $Param{UserID}, current_timestamp, $Param{UserID})";
     if ($Self->{DBObject}->Do(SQL => $SQL)) {
-      # log notice
-      $Self->{LogObject}->Log(
-          Priority => 'notice',
-          Message => "CustomerUser: '$Param{UserLogin}' created successfully ($Param{UserID})!",
-      );
-      # set password
-      if ($Param{UserPassword}) {
-          $Self->SetPassword(UserLogin => $Param{UserLogin}, PW => $Param{UserPassword});
-      }
-      return $Param{UserLogin};
+        # log notice
+        $Self->{LogObject}->Log(
+            Priority => 'notice',
+            Message => "CustomerUser: '$Param{UserLogin}' created successfully ($Param{UserID})!",
+        );
+        # set password
+        if ($Param{UserPassword}) {
+            $Self->SetPassword(UserLogin => $Param{UserLogin}, PW => $Param{UserPassword});
+        }
+        # cache resete
+        if ($Self->{CacheObject}) {
+            $Self->{CacheObject}->Delete(
+                Key => "CustomerUser::CustomerUserDataGet::User::$Param{UserLogin}",
+            );
+        }
+        return $Param{UserLogin};
     }
     else {
         return;
@@ -392,13 +541,15 @@ sub CustomerUserAdd {
 }
 
 sub CustomerUserUpdate {
-    my $Self = shift;
+    my $Self  = shift;
     my %Param = @_;
+
     # check ro/rw
     if ($Self->{ReadOnly}) {
         $Self->{LogObject}->Log(Priority => 'error', Message => "Customer backend is ro!");
         return;
     }
+
     # check needed stuff
     foreach my $Entry (@{$Self->{CustomerUserMap}->{Map}}) {
         if (!$Param{$Entry->[0]} && $Entry->[4] && $Entry->[0] ne 'UserPassword') {
@@ -406,6 +557,7 @@ sub CustomerUserUpdate {
             return;
         }
     }
+
     # check email address
     if ($Param{UserEmail} && !$Self->{CheckItemObject}->CheckEmail(Address => $Param{UserEmail})) {
         $Self->{LogObject}->Log(
@@ -417,14 +569,33 @@ sub CustomerUserUpdate {
     }
     # get old user data (pw)
     my %UserData = $Self->CustomerUserDataGet(User => $Param{ID});
+
+    # quote values
+    my %Value;
+    for my $Entry (@{$Self->{CustomerUserMap}->{Map}}) {
+        if ($Entry->[5] =~ /^int$/i) {
+            if ($Param{ $Entry->[0] }) {
+                $Value{ $Entry->[0] } = $Self->{DBObject}->Quote($Param{ $Entry->[0] }, 'Integer');
+            }
+            else {
+                $Value{ $Entry->[0] } = 0;
+            }
+        }
+        else {
+            if ($Param{ $Entry->[0] }) {
+                $Value{ $Entry->[0] } = "'" . $Self->{DBObject}->Quote($Param{$Entry->[0]}) . "'";
+            }
+            else {
+                $Value{ $Entry->[0] }= "''";
+            }
+        }
+    }
+
     # update db
     my $SQL = "UPDATE $Self->{CustomerTable} SET ";
     foreach my $Entry (@{$Self->{CustomerUserMap}->{Map}}) {
-        if ($Entry->[5] =~ /^int$/i) {
-            $SQL .= " $Entry->[2] = ".$Self->{DBObject}->Quote($Param{$Entry->[0]}).", ";
-        }
-        elsif ($Entry->[0] !~ /^UserPassword$/i) {
-            $SQL .= " $Entry->[2] = '".$Self->{DBObject}->Quote($Param{$Entry->[0]})."', ";
+        if ($Entry->[0] !~ /^UserPassword$/i) {
+            $SQL .= " $Entry->[2] = $Value{ $Entry->[0] }, ";
         }
     }
     $SQL .= " change_time = current_timestamp, ";
@@ -441,6 +612,12 @@ sub CustomerUserUpdate {
         if ($Param{UserPassword}) {
             $Self->SetPassword(UserLogin => $Param{UserLogin}, PW => $Param{UserPassword});
         }
+        # cache resete
+        if ($Self->{CacheObject}) {
+            $Self->{CacheObject}->Delete(
+                Key => "CustomerUser::CustomerUserDataGet::User::$Param{UserLogin}",
+            );
+        }
         return 1;
     }
     else {
@@ -465,6 +642,10 @@ sub SetPassword {
     my $CryptedPw = '';
     # md5 pw
     if ($Self->{ConfigObject}->Get('Customer::AuthModule::DB::CryptType') &&
+        $Self->{ConfigObject}->Get('Customer::AuthModule::DB::CryptType') eq 'plain') {
+        $CryptedPw = $Pw;
+    }
+    elsif ($Self->{ConfigObject}->Get('Customer::AuthModule::DB::CryptType') &&
         $Self->{ConfigObject}->Get('Customer::AuthModule::DB::CryptType') eq 'md5') {
         $CryptedPw = unix_md5_crypt($Pw, $Param{UserLogin});
     }
@@ -514,6 +695,12 @@ sub SetPassword {
                 Priority => 'notice',
                 Message => "CustomerUser: '$Param{UserLogin}' changed password successfully!",
             );
+            # cache resete
+            if ($Self->{CacheObject}) {
+                $Self->{CacheObject}->Delete(
+                    Key => "CustomerUser::CustomerUserDataGet::User::$Param{UserLogin}",
+                );
+            }
             return 1;
         }
         else {
