@@ -2,7 +2,7 @@
 # Kernel/System/Email.pm - the global email send module
 # Copyright (C) 2001-2007 OTRS GmbH, http://otrs.org/
 # --
-# $Id: Email.pm,v 1.20.2.1 2007/03/02 00:30:48 martin Exp $
+# $Id: Email.pm,v 1.29 2007/07/26 12:56:54 martin Exp $
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
 # the enclosed file COPYING for license information (GPL). If you
@@ -12,6 +12,7 @@
 package Kernel::System::Email;
 
 use strict;
+use warnings;
 use MIME::Words qw(:all);
 use MIME::Entity;
 use Mail::Address;
@@ -19,7 +20,7 @@ use Kernel::System::Encode;
 use Kernel::System::Crypt;
 
 use vars qw($VERSION);
-$VERSION = '$Revision: 1.20.2.1 $';
+$VERSION = '$Revision: 1.29 $';
 $VERSION =~ s/^\$.*:\W(.*)\W.+?$/$1/;
 
 =head1 NAME
@@ -42,25 +43,32 @@ create a object
 
     use Kernel::Config;
     use Kernel::System::Log;
+    use Kernel::System::Main;
     use Kernel::System::Time;
     use Kernel::System::DB;
     use Kernel::System::Email;
 
     my $ConfigObject = Kernel::Config->new();
-    my $LogObject    = Kernel::System::Log->new(
+    my $LogObject = Kernel::System::Log->new(
         ConfigObject => $ConfigObject,
     );
-    my $TimeObject    = Kernel::System::Time->new(
+    my $MainObject = Kernel::System::Main->new(
+        ConfigObject => $ConfigObject,
+        LogObject => $LogObject,
+    );
+    my $TimeObject = Kernel::System::Time->new(
         ConfigObject => $ConfigObject,
     );
     my $DBObject = Kernel::System::DB->new(
         ConfigObject => $ConfigObject,
+        MainObject => $MainObject,
         LogObject => $LogObject,
     );
     my $SendObject = Kernel::System::Email->new(
         ConfigObject => $ConfigObject,
         LogObject => $LogObject,
         DBObject => $DBObject,
+        MainObject => $MainObject,
         TimeObject => $TimeObject,
     );
 
@@ -79,21 +87,24 @@ sub new {
     # debug level
     $Self->{Debug} = $Param{Debug} || 0;
     # check all needed objects
-    foreach (qw(ConfigObject LogObject DBObject TimeObject)) {
+    foreach (qw(ConfigObject LogObject DBObject TimeObject MainObject)) {
         die "Got no $_" if (!$Self->{$_});
     }
-    # load generator backend module
-    my $GenericModule = $Self->{ConfigObject}->Get('SendmailModule')
-      || 'Kernel::System::Email::Sendmail';
-    if (!eval "require $GenericModule") {
-        die "Can't load sendmail backend module $GenericModule! $@";
-    }
-    # create backend object
-    $Self->{Backend} = $GenericModule->new(%Param);
-
     # create encode object
     $Self->{EncodeObject} = Kernel::System::Encode->new(%Param);
 
+    # load generator backend module
+    my $GenericModule = $Self->{ConfigObject}->Get('SendmailModule')
+        || 'Kernel::System::Email::Sendmail';
+    if (!$Self->{MainObject}->Require($GenericModule)) {
+        return;
+    }
+    # create backend object
+    $Self->{Backend} = $GenericModule->new(
+        %Param,
+        EncodeObject => $Self->{EncodeObject},
+    );
+
     $Self->{FQDN} = $Self->{ConfigObject}->Get('FQDN');
     $Self->{Organization} = $Self->{ConfigObject}->Get('Organization');
     $Self->{SendmailBcc} = $Self->{ConfigObject}->Get('SendmailBcc');
@@ -115,14 +126,14 @@ To send an email without already created header:
         Loop => 1, # not required, removes smtp from
         Attachment => [
             {
-              Filename    => "somefile.csv",
-              Content     => $ContentCSV,
-              ContentType => "text/csv",
+                Filename => "somefile.csv",
+                Content => $ContentCSV,
+                ContentType => "text/csv",
             }
             {
-              Filename    => "somefile.png",
-              Content     => $ContentPNG,
-              ContentType => "image/png",
+                Filename => "somefile.png",
+                Content => $ContentPNG,
+                ContentType => "image/png",
             }
         ],
         Sign => {
@@ -153,11 +164,12 @@ To send an email without already created header:
 sub Send {
     my $Self = shift;
     my %Param = @_;
-
     # check needed stuff
-    if (!$Param{Body}) {
-        $Self->{LogObject}->Log(Priority => 'error', Message => "Need Body!");
-        return;
+    foreach (qw(Body Charset)) {
+        if (!$Param{$_}) {
+            $Self->{LogObject}->Log(Priority => 'error', Message => "Need $_!");
+            return;
+        }
     }
     if (!$Param{To} && !$Param{Cc} && !$Param{Bcc}) {
         $Self->{LogObject}->Log(Priority => 'error', Message => "Need To, Cc or Bcc!");
@@ -230,14 +242,21 @@ sub Send {
     $Header{'X-Mailer'} = $Self->{ConfigObject}->Get('Product')." Mail Service (".$Self->{ConfigObject}->Get('Version').")";
     $Header{'X-Powered-By'} = 'OTRS - Open Ticket Request System (http://otrs.org/)';
     $Header{'Type'} = $Param{Type} || 'text/plain';
+    # define email encoding
     if ($Param{Charset} && $Param{Charset} =~ /utf(8|-8)/i) {
-#        $Header{'Encoding'} = '8bit';
-        $Header{'Encoding'} = 'base64';
+        $Header{'Encoding'} = '8bit';
     }
     else {
-#        $Header{'Encoding'} = '7bit';
         $Header{'Encoding'} = 'quoted-printable';
     }
+    # check if we need to force the encoding
+    if ($Self->{ConfigObject}->Get('SendmailEncodingForce')) {
+         $Header{'Encoding'} = $Self->{ConfigObject}->Get('SendmailEncodingForce');
+    }
+    # body encode if utf8 and base64 is used
+    if ($Header{'Encoding'} =~ /utf(8|-8)/i && $Header{'Encoding'} =~ /base64/i) {
+        $Self->{EncodeObject}->EncodeOutput(\$Param{Body});
+    }
     # check and create message id
     if ($Param{'Message-ID'}) {
         $Header{'Message-ID'} = $Param{'Message-ID'};
@@ -255,11 +274,8 @@ sub Send {
     if ($Self->{Organization}) {
         $Header{'Organization'} = encode_mimewords(Encode::encode($Param{Charset}, $Self->{Organization}), Charset => $Param{Charset}) || '';;
     }
-    # body encode
-    $Self->{EncodeObject}->EncodeOutput(\$Param{Body});
     # build MIME::Entity
     my $Entity = MIME::Entity->build(%Header, Data => $Param{Body});
-
     # add attachments to email
     if ($Param{Attachment}) {
         foreach my $Tmp (@{$Param{Attachment}}) {
@@ -270,8 +286,8 @@ sub Send {
                 # attach file to email
                 $Entity->attach(
                     Filename => $Upload{Filename},
-                    Data     => $Upload{Content},
-                    Type     => $Upload{ContentType},
+                    Data => $Upload{Content},
+                    Type => $Upload{ContentType},
                     Disposition => $Upload{Disposition} || 'inline',
                     Encoding => $Upload{Encoding} || 'base64',
                 );
@@ -285,6 +301,7 @@ sub Send {
             LogObject => $Self->{LogObject},
             DBObject => $Self->{DBObject},
             ConfigObject => $Self->{ConfigObject},
+            MainObject => $Self->{MainObject},
             CryptType => $Param{Sign}->{Type},
         );
         if (!$CryptObject) {
@@ -354,6 +371,7 @@ sub Send {
             LogObject => $Self->{LogObject},
             DBObject => $Self->{DBObject},
             ConfigObject => $Self->{ConfigObject},
+            MainObject => $Self->{MainObject},
             CryptType => $Param{Crypt}->{Type},
         );
         if (!$CryptObject) {
@@ -397,6 +415,7 @@ sub Send {
             LogObject => $Self->{LogObject},
             DBObject => $Self->{DBObject},
             ConfigObject => $Self->{ConfigObject},
+            MainObject => $Self->{MainObject},
             CryptType => $Param{Crypt}->{Type},
         );
         if (!$CryptObject) {
@@ -561,6 +580,8 @@ sub _MessageIDCreate {
 }
 1;
 
+=back
+
 =head1 TERMS AND CONDITIONS
 
 This software is part of the OTRS project (http://otrs.org/).
@@ -573,6 +594,6 @@ did not receive this file, see http://www.gnu.org/licenses/gpl.txt.
 
 =head1 VERSION
 
-$Revision: 1.20.2.1 $ $Date: 2007/03/02 00:30:48 $
+$Revision: 1.29 $ $Date: 2007/07/26 12:56:54 $
 
 =cut
