@@ -2,7 +2,7 @@
 # Kernel/System/Queue.pm - lib for queue functions
 # Copyright (C) 2001-2007 OTRS GmbH, http://otrs.org/
 # --
-# $Id: Queue.pm,v 1.62.2.1 2007/02/26 11:25:40 martin Exp $
+# $Id: Queue.pm,v 1.71 2007/06/28 21:47:59 martin Exp $
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
 # the enclosed file COPYING for license information (GPL). If you
@@ -12,12 +12,14 @@
 package Kernel::System::Queue;
 
 use strict;
+use warnings;
 use Kernel::System::StdResponse;
 use Kernel::System::Group;
 use Kernel::System::CustomerGroup;
+use Kernel::System::Valid;
 
 use vars qw($VERSION);
-$VERSION = '$Revision: 1.62.2.1 $';
+$VERSION = '$Revision: 1.71 $';
 $VERSION =~ s/^\$.*:\W(.*)\W.+?$/$1/;
 
 =head1 NAME
@@ -34,6 +36,37 @@ All queue functions. E. g. to add queue or other functions.
 
 =cut
 
+=item new()
+
+create a object
+
+    use Kernel::Config;
+    use Kernel::System::Log;
+    use Kernel::System::Main;
+    use Kernel::System::DB;
+    use Kernel::System::Queue;
+
+    my $ConfigObject = Kernel::Config->new();
+    my $LogObject = Kernel::System::Log->new(
+        ConfigObject => $ConfigObject,
+    );
+    my $MainObject = Kernel::System::Main->new(
+        ConfigObject => $ConfigObject,
+        LogObject => $LogObject,
+    );
+    my $DBObject = Kernel::System::DB->new(
+        MainObject => $MainObject,
+        ConfigObject => $ConfigObject,
+        LogObject => $LogObject,
+    );
+    my $QueueObject = Kernel::System::Queue->new(
+        ConfigObject => $ConfigObject,
+        LogObject => $LogObject,
+        DBObject => $DBObject,
+    );
+
+=cut
+
 sub new {
     my $Type = shift;
     my %Param = @_;
@@ -48,6 +81,7 @@ sub new {
     foreach (qw(DBObject ConfigObject LogObject)) {
         $Self->{$_} = $Param{$_} || die "Got no $_!";
     }
+    $Self->{ValidObject} = Kernel::System::Valid->new(%Param);
 
     # lib object
     $Self->{StdResponseObject} = Kernel::System::StdResponse->new(%Param);
@@ -70,7 +104,9 @@ sub new {
     # --------------------------------------------------- #
     $Self->{QueueDefaults} = {
         UnlockTimeout => 0,
-        EscalationTime => 0,
+        FirstResponseTime => 0,
+        UpdateTime => 0,
+        SolutionTime => 0,
         FollowUpLock => 0,
         SystemAddressID => 1,
         SalutationID => 1,
@@ -89,7 +125,7 @@ sub new {
 
 get a queue system email address as hash (Email, RealName)
 
-    my %Adresss = $Self->{QueueObject}->GetSystemAddress(
+    my %Adresss = $QueueObject->GetSystemAddress(
         QueueID => 123,
     );
 
@@ -122,7 +158,7 @@ sub GetSystemAddress {
 
 get a queue salutation
 
-    my $Salutation = $Self->{QueueObject}->GetSalutation(QueueID => 123);
+    my $Salutation = $QueueObject->GetSalutation(QueueID => 123);
 
 =cut
 
@@ -156,7 +192,7 @@ sub GetSalutation {
 
 get a queue signature
 
-    my $Signature = $Self->{QueueObject}->GetSignature(QueueID => 123);
+    my $Signature = $QueueObject->GetSignature(QueueID => 123);
 
 =cut
 
@@ -190,7 +226,7 @@ sub GetSignature {
 
 get std response of a queue
 
-    my $String = $Self->{QueueObject}->GetStdResponse(ID => 123);
+    my $String = $QueueObject->GetStdResponse(ID => 123);
 
 =cut
 
@@ -226,7 +262,7 @@ sub SetQueueStdResponse {
         $Self->{LogObject}->Log(Priority => 'error', Message => "Need ResponseID, QueueID and UserID!");
         return;
     }
-    if ($Self->QueueHasStdResponse(%Param)){
+    if ($Self->QueueHasStdResponse(%Param)) {
         return;
     }
     # db quote
@@ -236,7 +272,7 @@ sub SetQueueStdResponse {
     # sql
     my $SQL = sprintf(qq|INSERT INTO queue_standard_response (queue_id, standard_response_id, create_time, create_by, change_time, change_by)
     VALUES ( %s, %s, current_timestamp, %s, current_timestamp, %s)| , $Param{QueueID}, $Param{ResponseID}, $Param{UserID}, $Param{UserID});
-   # print "SQL was\n$SQL\n\n";
+
     if ($Self->{DBObject}->Do(SQL => $SQL)) {
         return 1;
     }
@@ -272,7 +308,7 @@ sub QueueHasStdResponse {
 
 get std responses of a queue
 
-    my %Responses = $Self->{QueueObject}->GetStdResponses(QueueID => 123);
+    my %Responses = $QueueObject->GetStdResponses(QueueID => 123);
 
 =cut
 
@@ -287,8 +323,7 @@ sub GetStdResponses {
     }
     # check if this result is present
     if ($Self->{"StdResponses::$Param{QueueID}"}) {
-        %StdResponses = %{$Self->{"StdResponses::$Param{QueueID}"}};
-        return %StdResponses;
+        return %{$Self->{"StdResponses::$Param{QueueID}"}};
     }
     # get std. responses
     my $SQL = "SELECT sr.id, sr.name ".
@@ -299,7 +334,7 @@ sub GetStdResponses {
         " AND ".
         " qsr.standard_response_id = sr.id".
         " AND ".
-        " sr.valid_id IN ( ${\(join ', ', $Self->{DBObject}->GetValidIDs())} )".
+        " sr.valid_id IN ( ${\(join ', ', $Self->{ValidObject}->ValidIDsGet())} )".
         " ORDER BY sr.name";
     $Self->{DBObject}->Prepare(SQL => $SQL);
     while (my @Row = $Self->{DBObject}->FetchrowArray()) {
@@ -315,11 +350,11 @@ sub GetStdResponses {
 
 get all system queues
 
-    my %Queues = $Self->{QueueObject}->GetAllQueues();
+    my %Queues = $QueueObject->GetAllQueues();
 
 get all system queues of a user with permission type (e. g. ro, move_into, rw, ...)
 
-    my %Queues = $Self->{QueueObject}->GetAllQueues(UserID => 123, Type => 'ro');
+    my %Queues = $QueueObject->GetAllQueues(UserID => 123, Type => 'ro');
 
 =cut
 
@@ -331,6 +366,9 @@ sub GetAllQueues {
     # fetch all queues
     my %MoveQueues;
     if ($Param{UserID}) {
+        if ($Self->{"QG::GetAllQueues::UserID::$Param{UserID}"}) {
+            return %{$Self->{"QG::GetAllQueues::UserID::$Param{UserID}"}};
+        }
         my @GroupIDs = $Self->{GroupObject}->GroupMemberList(
             UserID => $Param{UserID},
             Type => $Type,
@@ -342,7 +380,7 @@ sub GetAllQueues {
                 " WHERE ".
                 " group_id IN ( ${\(join ', ', @GroupIDs)} )".
                 " AND ".
-                " valid_id IN ( ${\(join ', ', $Self->{DBObject}->GetValidIDs())} )";
+                " valid_id IN ( ${\(join ', ', $Self->{ValidObject}->ValidIDsGet())} )";
             $Self->{DBObject}->Prepare(SQL => $SQL);
         }
         else {
@@ -350,6 +388,9 @@ sub GetAllQueues {
         }
     }
     elsif ($Param{CustomerUserID}) {
+        if ($Self->{"QG::GetAllQueues::CustomerUserID::$Param{CustomerUserID}"}) {
+            return %{$Self->{"QG::GetAllQueues::CustomerUserID::$Param{CustomerUserID}"}};
+        }
         my @GroupIDs = $Self->{CustomerGroupObject}->GroupMemberList(
             UserID => $Param{CustomerUserID},
             Type => $Type,
@@ -361,7 +402,7 @@ sub GetAllQueues {
                 " WHERE ".
                 " group_id IN ( ${\(join ', ', @GroupIDs)} )".
                 " AND ".
-                " valid_id IN ( ${\(join ', ', $Self->{DBObject}->GetValidIDs())} )";
+                " valid_id IN ( ${\(join ', ', $Self->{ValidObject}->ValidIDsGet())} )";
             $Self->{DBObject}->Prepare(SQL => $SQL);
         }
         else {
@@ -369,14 +410,28 @@ sub GetAllQueues {
         }
     }
     else {
+        if ($Self->{"QG::GetAllQueues"}) {
+            return %{$Self->{"QG::GetAllQueues"}};
+        }
         $Self->{DBObject}->Prepare(
             SQL => "SELECT id, name FROM queue WHERE valid_id IN ".
-                "( ${\(join ', ', $Self->{DBObject}->GetValidIDs())} )",
-            );
+                "( ${\(join ', ', $Self->{ValidObject}->ValidIDsGet())} )",
+        );
     }
     while (my @Row = $Self->{DBObject}->FetchrowArray()) {
         $MoveQueues{$Row[0]} = $Row[1];
     }
+
+    if ($Param{UserID}) {
+        $Self->{"QG::GetAllQueues::UserID::$Param{UserID}"} = \%MoveQueues;
+    }
+    elsif ($Param{CustomerUserID}) {
+        $Self->{"QG::GetAllQueues::CustomerUserID::$Param{UserID}"} = \%MoveQueues;
+    }
+    else {
+        $Self->{"QG::GetAllQueues"} = \%MoveQueues;
+    }
+
     return %MoveQueues;
 }
 
@@ -384,7 +439,7 @@ sub GetAllQueues {
 
 get all custom queues of one user
 
-    my %Queues = $Self->{QueueObject}->GetAllCustomQueues(UserID => 123);
+    my %Queues = $QueueObject->GetAllCustomQueues(UserID => 123);
 
 =cut
 
@@ -551,6 +606,9 @@ sub GetQueueGroupID {
         $Self->{LogObject}->Log(Priority => 'error', Message => "Need QueueID!");
         return;
     }
+    if ($Self->{"QG::GetQueueGroupID::$Param{QueueID}"}) {
+        return $Self->{"QG::GetQueueGroupID::$Param{QueueID}"};
+    }
     # db quote
     foreach (qw(QueueID)) {
         $Param{$_} = $Self->{DBObject}->Quote($Param{$_}, 'Integer');
@@ -566,6 +624,7 @@ sub GetQueueGroupID {
     while (my @Row = $Self->{DBObject}->FetchrowArray()) {
         $GID = $Row[0];
     }
+    $Self->{"QG::GetQueueGroupID::$Param{QueueID}"} = $GID;
     return $GID;
 }
 
@@ -574,17 +633,17 @@ sub GetQueueGroupID {
 add queue with attributes
 
     $QueueObject->QueueAdd(
-        Name            => 'Some::Queue',
-        ValidID         => 1,
-        GroupID         => 1,
+        Name => 'Some::Queue',
+        ValidID => 1,
+        GroupID => 1,
         SystemAddressID => 1,
-        SalutationID    => 1,
-        SignatureID     => 1,
-        UserID          => 123,
-        MoveNotify      => 0,
-        StateNotify     => 0,
-        LockNotify      => 0,
-        OwnerNotify     => 0,
+        SalutationID => 1,
+        SignatureID => 1,
+        UserID => 123,
+        MoveNotify => 0,
+        StateNotify => 0,
+        LockNotify => 0,
+        OwnerNotify => 0,
     );
 
 =cut
@@ -617,21 +676,26 @@ sub QueueAdd {
         SignatureID
         FollowUpID
         FollowUpLock
-        EscalationTime
+        FirstResponseTime
+        UpdateTime
+        SolutionTime
         MoveNotify
         StateNotify
         Comment
         ValidID
     );
 
-    for (qw(UnlockTimeout EscalationTime FollowUpLock SystemAddressID SalutationID SignatureID FollowUpID FollowUpLock MoveNotify StateNotify LockNotify OwnerNotify DefaultSignKey Calendar)) {
+    for (qw(UnlockTimeout FirstResponseTime UpdateTime SolutionTime
+        FollowUpLock SystemAddressID SalutationID SignatureID
+        FollowUpID FollowUpLock MoveNotify StateNotify LockNotify OwnerNotify DefaultSignKey Calendar)) {
         # these are coming from Config.pm
         # I added default values in the Load Routine
         $Param{$_} = $Self->{QueueDefaults}{$_} || 0  unless ($Param{$_});
     };
 
     # check needed stuff
-    foreach (qw(Name GroupID SystemAddressID SalutationID SignatureID MoveNotify StateNotify LockNotify OwnerNotify ValidID UserID)) {
+    foreach (qw(Name GroupID SystemAddressID SalutationID SignatureID MoveNotify StateNotify
+        LockNotify OwnerNotify ValidID UserID)) {
         if (!defined($Param{$_})) {
             $Self->{LogObject}->Log(Priority => 'error', Message => "Need $_!");
             return;
@@ -658,7 +722,9 @@ sub QueueAdd {
     foreach (qw(Name DefaultSignKey Calendar Comment)) {
         $Param{$_} = $Self->{DBObject}->Quote($Param{$_}) || '';
     };
-    foreach (qw(GroupID UnlockTimeout SystemAddressID SalutationID SignatureID FollowUpID FollowUpLock EscalationTime MoveNotify StateNotify ValidID)) {
+    foreach (qw(GroupID UnlockTimeout SystemAddressID SalutationID SignatureID FollowUpID FollowUpLock
+        FirstResponseTime UpdateTime SolutionTime
+        MoveNotify StateNotify ValidID)) {
         $Param{$_} = $Self->{DBObject}->Quote($Param{$_}, 'Integer');
     }
     my $SQL = "INSERT INTO queue ".
@@ -670,7 +736,9 @@ sub QueueAdd {
         " default_sign_key, ".
         " salutation_id, ".
         " signature_id, ".
-        " escalation_time, ".
+        " first_response_time, ".
+        " update_time, ".
+        " solution_time, ".
         " follow_up_id, ".
         " follow_up_lock, ".
         " state_notify, ".
@@ -692,7 +760,9 @@ sub QueueAdd {
         " '$Param{DefaultSignKey}', ".
         " $Param{SalutationID}, ".
         " $Param{SignatureID}, ".
-        " $Param{EscalationTime}, ".
+        " $Param{FirstResponseTime}, ".
+        " $Param{UpdateTime}, ".
+        " $Param{SolutionTime}, ".
         " $Param{FollowUpID}, ".
         " $Param{FollowUpLock}, ".
         " $Param{StateNotify}, ".
@@ -742,10 +812,10 @@ sub QueueAdd {
             }
         }
         return $QueueID;
-   }
-   else {
+    }
+    else {
         return;
-   }
+    }
 }
 
 =item QueueGet()
@@ -754,10 +824,12 @@ get queue attributes
 
     my %Queue = $QueueObject->QueueGet(
         ID => 123,
+        Cache => 1, # optional
     );
 
     my %Queue = $QueueObject->QueueGet(
         Name => 'Some::Queue',
+        Cache => 1, # optional
     );
 
 =cut
@@ -780,7 +852,8 @@ sub QueueGet {
     # sql
     my $SQL = "SELECT q.name, q.group_id, q.unlock_timeout, ".
         " q.system_address_id, q.salutation_id, q.signature_id, q.comments, q.valid_id, ".
-        " q.escalation_time, q.follow_up_id, q.follow_up_lock, sa.value0, sa.value1, q.id, ".
+        " q.first_response_time, q.update_time, q.solution_time, ".
+        " q.follow_up_id, q.follow_up_lock, sa.value0, sa.value1, q.id, ".
         " q.move_notify, q.state_notify, q.lock_notify, q.owner_notify, q.default_sign_key, ".
         " q.calendar_name ".
         " FROM ".
@@ -803,26 +876,28 @@ sub QueueGet {
     $Self->{DBObject}->Prepare(SQL => $SQL);
     while (my @Data = $Self->{DBObject}->FetchrowArray()) {
         %QueueData = (
-            QueueID => $Data[13],
+            QueueID => $Data[15],
             Name => $Data[0],
             GroupID => $Data[1],
             UnlockTimeout => $Data[2],
-            EscalationTime => $Data[8],
-            FollowUpID => $Data[9],
-            FollowUpLock => $Data[10],
+            FirstResponseTime => $Data[8],
+            UpdateTime => $Data[9],
+            SolutionTime => $Data[10],
+            FollowUpID => $Data[11],
+            FollowUpLock => $Data[12],
             SystemAddressID => $Data[3],
             SalutationID => $Data[4],
             SignatureID => $Data[5],
             Comment => $Data[6],
             ValidID => $Data[7],
-            Email => $Data[11],
-            RealName => $Data[12],
-            StateNotify => $Data[15],
-            MoveNotify => $Data[14],
-            LockNotify => $Data[16],
-            OwnerNotify => $Data[17],
-            DefaultSignKey => $Data[18],
-            Calendar => $Data[19],
+            Email => $Data[13],
+            RealName => $Data[14],
+            StateNotify => $Data[17],
+            MoveNotify => $Data[16],
+            LockNotify => $Data[18],
+            OwnerNotify => $Data[19],
+            DefaultSignKey => $Data[20],
+            Calendar => $Data[21],
         );
         $Self->{"QG::$Suffix$Param{What}"} = \%QueueData;
     }
@@ -843,18 +918,18 @@ sub QueueGet {
 update queue attributes
 
     $QueueObject->QueueUpdate(
-        QueueID         => 123,
-        Name            => 'Some::Queue',
-        ValidID         => 1,
-        GroupID         => 1,
+        QueueID => 123,
+        Name => 'Some::Queue',
+        ValidID => 1,
+        GroupID => 1,
         SystemAddressID => 1,
-        SalutationID    => 1,
-        SignatureID     => 1,
-        UserID          => 123,
-        MoveNotify      => 0,
-        StateNotify     => 0,
-        LockNotify      => 0,
-        OwnerNotify     => 0,
+        SalutationID => 1,
+        SignatureID => 1,
+        UserID => 123,
+        MoveNotify => 0,
+        StateNotify => 0,
+        LockNotify => 0,
+        OwnerNotify => 0,
     );
 
 =cut
@@ -863,7 +938,8 @@ sub QueueUpdate {
     my $Self = shift;
     my %Param = @_;
     # check needed stuff
-    foreach (qw(QueueID Name ValidID GroupID SystemAddressID SalutationID SignatureID FollowUpID UserID MoveNotify StateNotify LockNotify OwnerNotify)) {
+    foreach (qw(QueueID Name ValidID GroupID SystemAddressID SalutationID SignatureID FollowUpID UserID
+        MoveNotify StateNotify LockNotify OwnerNotify)) {
         if (!defined ($Param{$_})) {
             $Self->{LogObject}->Log(Priority => 'error', Message => "Need $_!");
             return;
@@ -878,13 +954,17 @@ sub QueueUpdate {
     # db quote
     my %DB = ();
     # check !!!
-    foreach (qw(UnlockTimeout EscalationTime FollowUpLock MoveNotify StateNotify LockNotify OwnerNotify FollowUpID FollowUpLock DefaultSignKey Calendar)) {
+    foreach (qw(UnlockTimeout FollowUpLock MoveNotify StateNotify LockNotify OwnerNotify
+        FirstResponseTime UpdateTime SolutionTime
+        FollowUpID FollowUpLock DefaultSignKey Calendar)) {
         $Param{$_} = 0 if (!$Param{$_});
     }
     foreach (qw(Name DefaultSignKey Calendar Comment)) {
         $DB{$_} = $Self->{DBObject}->Quote($Param{$_}) || '';
     };
-    foreach (qw(QueueID GroupID UnlockTimeout SystemAddressID SalutationID SignatureID FollowUpID FollowUpLock EscalationTime StateNotify LockNotify OwnerNotify MoveNotify StateNotify ValidID UserID)) {
+    foreach (qw(QueueID GroupID UnlockTimeout SystemAddressID SalutationID SignatureID FollowUpID
+        FirstResponseTime UpdateTime SolutionTime
+        FollowUpLock StateNotify LockNotify OwnerNotify MoveNotify StateNotify ValidID UserID)) {
         $DB{$_} = $Self->{DBObject}->Quote($Param{$_}, 'Integer');
     }
     # cleanup queue name
@@ -918,7 +998,9 @@ sub QueueUpdate {
         " comments = '$DB{Comment}', " .
         " group_id = $DB{GroupID}, " .
         " unlock_timeout = $DB{UnlockTimeout}, " .
-        " escalation_time = $DB{EscalationTime}, " .
+        " first_response_time = $DB{FirstResponseTime}, " .
+        " update_time = $DB{UpdateTime}, " .
+        " solution_time = $DB{SolutionTime}, " .
         " follow_up_id = $DB{FollowUpID}, " .
         " follow_up_lock = $DB{FollowUpLock}, " .
         " system_address_id = $DB{SystemAddressID}, " .
@@ -963,8 +1045,11 @@ sub QueueUpdate {
         return;
     }
 }
+
 1;
 
+=back
+
 =head1 TERMS AND CONDITIONS
 
 This Software is part of the OTRS project (http://otrs.org/).
@@ -977,6 +1062,6 @@ did not receive this file, see http://www.gnu.org/licenses/gpl.txt.
 
 =head1 VERSION
 
-$Revision: 1.62.2.1 $ $Date: 2007/02/26 11:25:40 $
+$Revision: 1.71 $ $Date: 2007/06/28 21:47:59 $
 
 =cut
